#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <sys/time.h>
#include <getopt.h>
#include <pwd.h>
#include <sys/stat.h>

#include "vvl_defs.h"
#include "pport.h"
#include "cmd.h"
#include "cpia.h"
#include "convert.h"
#include "keys.h"
#include "picture_coder.h"
#include "internet.h"
#include "mcam.h"

#include "mcam_proto.h"

#define USE_X
#ifdef NO_X
#undef USE_X
#endif

int debug_flag;
int color_depth_mode;
int color_depth;
int coding_quality;
unsigned char *yuvbufptr;
int	y_difference, u_difference, v_difference, treshhold;
int apcor_values[4];

/* number of seconds to test for stream ready in get_a_frame() */
#define MAX_WAIT_TIME 10

int test_flag;

int port_address;
int window_open_flag = 0;
char *home_dir;
char *mcamrc_dir;

int picture_format;
int old_picture_format;
int save_frames_flag;
int save_frames_format;
int start_frame;
int frame_count;
int frame_delay;
char *frame_dir;
char *server_password;
char *remote_password;
char *server_control_password;
int coding_mode;/* none, mpeg, jpeg or wavelet coding decoding */
int current_window_size;
int output_to_frame_flag;
int special_effects_flag;
int store_only_on_change_flag;
int gain_ceiling;
int man_exposure_flag;
/* for color balance */
int color_balance_mode;
char *command_line_preset_filename;
int clock_position;
int use_utc_flag;
int show_time_flag;
char screen_text[64];
int text_to_screen_flag;
int text_x, text_y;
int text_fr, text_fg, text_fb, text_br, text_bg, text_bb;
int selected_font;
int sensor_frames_per_second;

BOOL CheckForCPIA( BYTE PortNumber );
BOOL DoSystemTest( void );
BOOL UploadTestData( WORD DummyBlocks );
BOOL TestEEPROM( void );


void UploadAndSaveDRAM( void );
void UploadBlock( void );
void DoUploadImage( void );


BYTE gPortNumber;
BYTE __huge *gStreamBuffer;
BYTE __huge *DataBuffer;

BOOL gAEC = TRUE;

BYTE gFW_Ver, gFW_Rev, gVC_Ver, gVC_Rev;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc(strlen(s) +  1);
if(p) strcpy(p, s);
return(p);
}/* end function strsave */


int convert(\
unsigned char *ibuf, int ibufsize, unsigned char *obuf, int type)
{
if(debug_flag)
	{
	fprintf(stderr,\
	"convert(): ibuf=%lu ibufsize=%d obuf=%lu type=%d\n",\
	(long)ibuf, ibufsize, (long)obuf, type);
	}

if(ibuf[COMP_ENABLE] == COMPRESSED)
	{
	if(debug_flag)
		{
		fprintf(stderr, "Compressed type=%d\n", type);
		}
	switch (type)
		{
		case 15:return stream2rgb15c(ibuf+64,obuf,ibufsize-64);
		case 16:return stream2rgb16c(ibuf+64,obuf,ibufsize-64);
		case 24:return stream2rgb24c(ibuf+64,obuf,ibufsize-64);
		case 32:return stream2rgb32c(ibuf+64,obuf,ibufsize-64);
		default: fprintf(stderr, "illegal convertion type\n");
		}
	}
else
	{
	if(debug_flag)
		{
		fprintf(stderr, "Uncompressed type=%d\n", type);
		}
	switch (type)
		{
		case 15:return stream2rgb15(ibuf+64,obuf,ibufsize-64);
		case 16:return stream2rgb16(ibuf+64,obuf,ibufsize-64);
		case 24:return stream2rgb24(ibuf+64,obuf,ibufsize-64);
		case 32:return stream2rgb32(ibuf+64,obuf,ibufsize-64);
		default: fprintf(stderr, "illegal convertion type\n");
		}
	}
return 0;
}


int main(int argc, char *argv[])
{
int a, b, c, i;
float fa, fb;
int tmp, ch;
DWORD dw_tmp;
DWORD actual_bytes;
BOOL rtn;
BYTE sys, vp, stream, fatal, cmd, vp_stat;
BYTE cam_setup0, cam_status;
FILE *fptr;
int image_size;
unsigned char *ucptr;
struct passwd *userinfo;
int setup_file_present_flag;
char temp[512];
char path_filename[1204];
int milliseconds_delay;
int compression_flag;
time_t start_time, end_time, elapsed_seconds;
float frames_per_second;
int frames_displayed;
/* for exposure */
int coarse_exposure;
int current_coarse_exposure;
int gain, fine_exposure, coarse_exposure_low, coarse_exposure_high;
int current_gain, current_fine_exposure;
int current_coarse_exposure_low, current_coarse_exposure_high;
int red_compensation;
int green_1_compensation, green_2_compensation;
int blue_compensation;
int exposure_mode, compute_mode, centre_weight;
int red_gain, green_gain, blue_gain;
int current_red_gain, current_green_gain, current_blue_gain;
int brightness, contrast, saturation;
char *internet_address;
int internet_port;
int cmd_line_client_flag;
int cmd_line_port_address_flag;
int cmd_line_server_flag;
int cmd_line_internet_port_flag;
char *cmd_line_internet_address;
char *cmd_line_internet_password;
int cmd_line_coding_mode;
FILE *cfptr;
//For longer exposures (top sensor 2.56 sec, bottom sensor 0.32 sec)
// added by NDJ
int grabmode;  
int ccddivide; 
int baseclk;   
int framerate; 

/*  con io 1 byte at the time */
setbuf(stderr, NULL);
setbuf(stdin, NULL);
 
fprintf(stderr, "\n\n   ======   mcam-%s  by Panteltje   =====\n\n", VERSION);

/* some defaults, reading from setup or cpia may change these */
window_open_flag = 0;
current_window_size = UNKNOWN;
gPortNumber = 1;
save_frames_flag = 0;
save_frames_format = 1;/* .ppm by default */
start_frame = 0;
frame_count = 0;
frame_delay = 0;
milliseconds_delay = 0;
picture_format = 1;
old_picture_format = 1;/* big size */
color_balance_mode = 1;/* auto */
exposure_mode = 0; /* no change */
compute_mode = 0; /* no change */
centre_weight = 0; /* no change */
gain_ceiling = 0; /* lowest gain */
gain = 0; /*  */
fine_exposure = 1;
coarse_exposure = 511; /* get a picture, when dark, for user confidence */
red_compensation = 220;
green_1_compensation = 214;
green_2_compensation = 214;
blue_compensation = 230;
contrast = 50;
brightness = 50;
saturation = 50;
remote_password = 0;
color_depth = 0;
color_depth_mode = 0;/* auto */
coding_mode = NO_CODING;
coding_quality = 35;/* percent the lower, the more compression, use 25 to 50 */
output_to_frame_flag = 0;
special_effects_flag = 0;
command_line_preset_filename = NULL;
clock_position = BOTTOM_MIDDLE;
show_time_flag = 0;
use_utc_flag = 1;
text_to_screen_flag = 0;
screen_text[0] = 0;

text_x = 0;
text_y = 0;
text_fr = 255;
text_fg = 255;
text_fb = 255;
text_br = 0;
text_bg = 0;
text_bb = 0; 
selected_font = S14_FONT;
red_gain = 0;
green_gain = 0;
blue_gain = 0;
/*
apertur correction (affects image sharpness and noise)
4 most sign. bits treshold, 4 least sign. bits intensity
*/
apcor_values[0] = 0x2c;
apcor_values[1] = 0x2a;
apcor_values[2] = 0x4d;
apcor_values[3] = 0x4a;

// added by NDJ for loner exposures
ccddivide = 0;
baseclk = 1;
framerate = 0;

/* process command line arguments */
test_flag = 0;
cmd_line_internet_address = 0;
cmd_line_internet_password = 0;
debug_flag = 0;
port_address = 0x378;
internet_port = 6668;
cmd_line_server_flag = 0;
cmd_line_client_flag = 0;
cmd_line_internet_port_flag = 0;
cmd_line_coding_mode = 0;
mcamrc_dir = 0;
while(1)
	{
	a = getopt(argc, argv, "a:cehl:mOp:str:u:w:y");
	if(a == -1) break;
	switch(a)
		{
        case 'a':
        	sscanf(optarg, "%500s", temp);
        	cmd_line_internet_address = strsave(temp);
        	if(! cmd_line_internet_address)
        		{
        		fprintf(stderr,\
        		"cannot allocate space for cmd_line_internet_address\n");
        		exit(1);
        		}
        	break;
        case 'c':
        	cmd_line_client_flag = 1;
        	break;
        case 'e': /* not to conflict with -d display option for x */
			debug_flag = 1;
			break;
		case 'h':
			fprintf(stderr,\
"Usage:\n\
mcam [-a IP address] [-c] [-d] [-h] [-l LPT address] [-p IP port] [-s]\n\
[-u presetfile] [-w remote password]\n\
\t-a  IP address for server you connect to (example localhost).\n\
\t-c  start as internet client, no camera operations are done.\n\
\t-e  print debug messages (o.a. function calls and arguments).\n\
\t-h  help (this help).\n\
\t-l  hardware printer port address (in hex) default is 0x378.\n\
\t-m  use mpeg coding (decoding will follow automatically).\n\
\t-O  wait for framedir/mcam.ppm.ready to be erased,\n\
\t    then save current frame in framedir/mcam.ppm,\n\
\t    then create new framedir/mcam.ppm.ready.\n\
\t-p  IP port (default 6668).\n\
\t-r  mcamrc_dir (where to look for / .mcamrc (default is homedir) ).\n\
\t-s  start as internet server.\n\
\t-u  filename  (load preset file framedir/p.filename (client only) ).\n\
\t-w  password for server you connect to.\n\
");
			exit(0);	
			break;
		case 'l':
			sscanf(optarg, "%x", &port_address);
			break;
		case 'm':/* use mpeg coding (decoding will folow automatic) */
			cmd_line_coding_mode = 1;
			break;
		case 'O':
			output_to_frame_flag = 1;
			break;
		case 'p':
			cmd_line_internet_port_flag = 1;
			sscanf(optarg, "%d", &internet_port);
			break;
		case 'r':
        	sscanf(optarg, "%500s", temp);
        	mcamrc_dir = strsave(temp);
        	if(! mcamrc_dir)
        		{
        		fprintf(stderr,\
        		"cannot allocate space for mcamrc_dir\n");
        		exit(1);
        		}
			break;			
		case 's':
			cmd_line_server_flag = 1;
			break;
		case 't':
			test_flag = 1;
			break;
		case 'u':
			sscanf(optarg, "%100s", temp);
			command_line_preset_filename = strsave(temp);
			if(! command_line_preset_filename)
				{
				fprintf(stderr, "Cannot allocate space for command_line_preset_filename\n");
				exit(1);
				}			
			break;
		case 'w':
			sscanf(optarg, "%15s", temp);
			cmd_line_internet_password = strsave(temp);
			if(! cmd_line_internet_password)
				{
				fprintf(stderr,\
				"Cannot allocate space for cmd_line_internet_password\n");
				exit(1);
				}
			break;
// 		case 'y': /* frames in yuv format to stderr */
//			save_frames_format = 2;
// 			set_setup("save_frames_format", "yuv");
// 			break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			exit(1);
			break;			
		default:
			break;
		}/* end switch a */
	}/* end while getopt() */

if(! cmd_line_client_flag)
	{
	/* set io permissions, need full range for ECP negotiation */
	a = iopl(3);
	if(a != 0)
		{
		fprintf(stderr,\
		"Cannot get IO permission, are you root? iopl() returned %d\n", a);
		exit(1);
		}
	}

/* create a buffer, data via internet goes here */
DataBuffer = malloc(STREAM_BUFFER_LEN + HEADER_SIZE);
if(! DataBuffer)
	{
	fprintf(stderr, "Cannot allocate space for DataBuffer\n");
	exit(1);
	}

gStreamBuffer = DataBuffer + HEADER_SIZE;/* cpia rx buffer starts here */

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stderr, "Cannot allocate space for home_dir\n");
	exit(1);
	}

/* defaults, get from cpia if in chip */

/* if no mcamrc_dir specified on command line, use user's home directory */
if(! mcamrc_dir)
	{
	/* default directory to save frames */
	mcamrc_dir = strsave(home_dir);
	if(! mcamrc_dir)
		{
		fprintf(stderr, "Cannot allocate space for mcamrc_dir\n");
		exit(1);
		}
	}/* end if ! mcamrc_dir */

internet_address = strsave("localhost");
if(! internet_address)
	{
	fprintf(stderr, "Cannot allocate space for internet_address\n");
	exit(1);
	}

server_password = strsave("no");
if(! server_password)
	{
	fprintf(stderr, "cannot allocate space for local_pasword\n");
	exit(1);
	}

server_control_password = strsave("no");
if(! server_control_password)
	{
	fprintf(stderr, "cannot allocate space for server_control_password\n");
	exit(1);
	}

sensor_frames_per_second = 3;
 
/* end defaults */

if(! cmd_line_client_flag)
	{
	if ( !CLINK_Init() )
		{
		fprintf(stderr, "PP_TEST: CLINK_Init() failed - exiting\n" );
		exit( 1 );
		}

	if ( !CheckForCPIA( gPortNumber ) )
		{
   		fprintf(stderr, "mcam: unable to find DUAL on LPT%d\n", gPortNumber );
		if ( gPortNumber == 1 ) gPortNumber = 2;
		else gPortNumber = 1;
		if ( !CheckForCPIA( gPortNumber ) )
		   	{
		   	fprintf(stderr, "mcam: unable to find DUAL on LPT%d\n", gPortNumber );
			}
		}

	CPIA_SetECPTiming(1);

	}/* end if not cmd_line_client_flag */

setup_file_present_flag = load_setup(home_dir);
if(setup_file_present_flag)
	{
	/*
	frames per second, and this automatically limits the exposre time ??
	*/
	if( get_setup("sensor_frames_per_second", temp, 0) )
		{
		sensor_frames_per_second = atoi(temp);

		} /* end if sensor_frames_per_second */
		
	/* exposure */
	if( get_setup("auto_exposure_control", temp, 0) )
		{
		if(strncasecmp(temp, "on", 2) == 0)
			{
			man_exposure_flag = 1;
			CPIA_SetExposure(0, 2, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
			}
		else /* off */
			{
			man_exposure_flag = 0;
			CPIA_SetExposure(0, 3, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
			}
		}
	if( get_setup("fine_exposure", temp, 0) )
		{
		fine_exposure = atoi(temp);
		}
	if( get_setup("coarse_exposure", temp, 0) )
		{
		coarse_exposure = atoi(temp);
		}
	if( get_setup("gain", temp, 0) )
		{
		gain = atoi(temp);
		}
	if( get_setup("red_compensation", temp, 0) )
		{
		red_compensation = atoi(temp);
		}
	if( get_setup("green_1_compensation", temp, 0) )
		{
		green_1_compensation = atoi(temp);
		}
	if( get_setup("green_2_compensation", temp, 0) )
		{
		green_2_compensation = atoi(temp);
		}
	if( get_setup("blue_compensation", temp, 0) )
		{
		blue_compensation = atoi(temp);
		}

	/* calculate derived values */
	coarse_exposure_low = coarse_exposure % 256;
	coarse_exposure_high = coarse_exposure / 256;

	CPIA_SetExposure(0, 1, 1, 0,\
	gain, fine_exposure, coarse_exposure_low, coarse_exposure_high,\
	red_compensation, green_1_compensation, green_2_compensation,\
	blue_compensation);
	/* end exposure */
		
	/* brightness, contrast and saturation */
	if( get_setup("brightness", temp, 0) )
		{
		brightness = atoi(temp);
		}

	if( get_setup("contrast", temp, 0) )
		{
		contrast = atoi(temp);
		}			

	if(	get_setup("saturation", temp, 0) )
		{
		saturation = atoi(temp);
		}			

	CPIA_SetColourParams(brightness, contrast, saturation);
	/* end brightness, contrast, saturation */

	/* frame directory */
	if( get_setup("frame_dir", temp, 0) )
		{
		free(frame_dir);
		frame_dir = strsave(temp);
		if(! frame_dir)
			{
			fprintf(stderr, "cannot allocate space for frame_dir\n");
			exit(1);
			}
		}
	/* end frame directory */
	
	/* compression */
	if( get_setup("compression", temp, 0) )
		{
		if(strncasecmp(temp, "on", 2) == 0) compression_flag = 1;
		else compression_flag = 0;
		CPIA_SetCompression(compression_flag, 0);
		}
	/* end compression */

	/* size */
	if( get_setup("size", temp, 0) )
		{
		if(strncasecmp(temp, "small", 5) == 0) picture_format = 0;
		else picture_format = 1;
		CPIA_SetFormat(picture_format, 1, 0);
		}
	/* end size */

	/* gain_ceiling  */
	if(get_setup("gain_ceiling", temp, 0) )
		{
		gain_ceiling = atoi(temp);

		CPIA_SetExposure(gain_ceiling, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
		}
	/* end gain_ceiling */

	/* color balance */
	if(get_setup("red_gain", temp, 0) )
		{
		red_gain = atoi(temp);
		}
	if(get_setup("green_gain", temp, 0) )
		{
		green_gain = atoi(temp);
		}
	if(get_setup("blue_gain", temp, 0) )
		{
		blue_gain = atoi(temp);
		}
	if(get_setup("color_balance_mode", temp, 0) )
		{
		if(strncasecmp(temp, "auto", 4) == 0) color_balance_mode = 1;
		else color_balance_mode = 0;
		}
	if(color_balance_mode == 0)/* manual */
		{
		/* set auto off */
		CPIA_SetColourBalance(3, 0, 0, 0);

		/* set values */
		CPIA_SetColourBalance(1, red_gain, green_gain, blue_gain);
		}
	else /* auto */
		{
		/* auto on */
		CPIA_SetColourBalance(2, 0, 0, 0);
		}
	/* end color balance */
		
	/* format that frames are stored in */
	if( get_setup("save_frames_format", temp, 0) )
		{
		save_frames_format = 0;
		if(strncasecmp(temp, "yuv", 3) == 0) save_frames_format = 2;
		else if(strncasecmp(temp, ".ppm", 4) == 0) save_frames_format = 1;
		else if(strncasecmp(temp, "xbuffer", 7) == 0) save_frames_format = 0;
		}
	/* end format that frames are stored in */
	
	/* store frames on or off */
	if(get_setup("save_frames", temp, 0) )
		{
		if(strncasecmp(temp, "on", 2) == 0) save_frames_flag = 1;
		else save_frames_flag = 0;
		}
	/* end store frames on or off */

	/* start frame */
	if(get_setup("save_frames_start_frame", temp, 0) )
		{
		start_frame = atoi(temp);		
		frame_count = start_frame;
		}
	/* end start frame */

	/* frame delay */
	if(get_setup("save_frames_frame_delay", temp, 0) )
		{
		frame_delay = atoi(temp);
		}
	/* end frame _delay */

	/* store only on chage */
	if(get_setup("store_only_on_change", temp, 0) )
		{
		if(strncasecmp(temp, "yes", 3) == 0)
			{
			store_only_on_change_flag = 1;
			}
		else store_only_on_change_flag = 0;
		}
	/* end store only on change */
	
	/* internet location */
	if(get_setup("internet_address", temp, 0) )
		{
		internet_address = strsave(temp);
		if(! internet_address) 
			{
			fprintf(stderr, "cannot malloc space for internet_address\n");			
			exit(1);
			}
		}
	if(! cmd_line_internet_port_flag)
		{
		if(get_setup("internet_port", temp, 0) )
			{
			internet_port = atoi(temp);
			}
		}
	/* end internet location */
	
	/* passwords */
	if(get_setup("server_password", temp, 0) )
		{
		free(server_password);
		server_password = strsave(temp);
		if(! server_password)
			{
			fprintf(stderr,\
			"Could not allocate space for server_password\n");
			exit(1);
			}
		}
	if(get_setup("server_control_password", temp, 0) )
		{
		free(server_control_password);
		server_control_password = strsave(temp);
		if(! server_control_password)
			{
			fprintf(stderr,\
			"Could not allocate space for server_control_password\n");
			exit(1);
			}
		}
	/* end passwords */

	/* color depth */
	if(get_setup("color_depth_mode", temp, 0) )
		{
		color_depth_mode = atoi(temp);		
		}

	if( get_setup("color_depth", temp, 0) )
		{
		color_depth = atoi(temp);
		}
	/* end color depth */

	/* coding */
	if(get_setup("coding", temp, 0) )
		{
		if(strncmp(temp, "off", 3) == 0)
			{
			coding_mode = NO_CODING;
			}
		else if(strncmp(temp, "jpeg", 4) == 0)
			{
			coding_mode = JPEG_CODING;
			}
		else if(strncmp(temp, "mpeg", 4) == 0)
			{
			coding_mode = MPEG2_CODING;
			}
		else if(strncmp(temp, "wavelet", 7) == 0)
			{
			coding_mode = WAVELET_CODING;
			}
		}	
	/* end coding */

	/* jpeg, mpeg, wavelet coding quality */
	if(get_setup("coding_quality", temp, 0) )
		{
		coding_quality = atoi(temp);
		}
	/* end coding quality */

	/* show time */
	if(get_setup("show_time", temp, 0) )
		{
		if(strncasecmp(temp, "yes", 3) == 0) show_time_flag = 1;
		else show_time_flag = 0;
		}
	/* end show time */

	/* clock position */
	if(get_setup("clock_position", temp, 0) )
		{
		if(strncasecmp(temp, "top_left", 8) == 0)
			{
			clock_position = TOP_LEFT;
			}
		if(strncasecmp(temp, "top_middle", 10) == 0)
			{
			clock_position = TOP_MIDDLE;
			}
		if(strncasecmp(temp, "top_right", 9) == 0)
			{
			clock_position = TOP_RIGHT;
			}
		if(strncasecmp(temp, "bottom_left", 11) == 0)
			{
			clock_position = BOTTOM_LEFT;
			}
		if(strncasecmp(temp, "bottom_middle", 13) == 0)
			{
			clock_position = BOTTOM_MIDDLE;
			}
		if(strncasecmp(temp, "botom_right", 12) == 0)
			{
			clock_position = BOTTOM_RIGHT;
			}
		}
	/* end clock position */

	/* UTC time */
	if(get_setup("time", temp, 0) )
		{
		if(strncasecmp(temp, "utc", 3) == 0) use_utc_flag = 1;
		else use_utc_flag = 0;		
		}
	/* end UTC time */

	/* font */
	if(get_setup("font", temp, 0) )
		{
		if(strcasecmp(temp, "s14font") == 0) selected_font = S14_FONT;
		else if (strcasecmp(temp, "txtfont") == 0) selected_font = TXT_FONT;
		}
	/* end font */

	/* apertur_correction */
	if(get_setup("apertur_treshhold_1", temp, 0) )
		{
		a = atoi(temp);
		if(get_setup("apertur_level_1", temp, 0) )
			{
			b = atoi(temp);
			apcor_values[0] = ( (a & 0xf) << 4) + (b & 0xf); 		
			}
		}
	if(get_setup("apertur_treshhold_2", temp, 0) )
		{
		a = atoi(temp);
		if(get_setup("apertur_level_2", temp, 0) )
			{
			b = atoi(temp);
			apcor_values[1] = ( (a & 0xf) << 4) + (b & 0xf); 		
			}
		}
	if(get_setup("apertur_treshhold_3", temp, 0) )
		{
		a = atoi(temp);
		if(get_setup("apertur_level_3", temp, 0) )
			{
			b = atoi(temp);
			apcor_values[2] = ( (a & 0xf) << 4) + (b & 0xf); 		
			}
		}
	if(get_setup("apertur_treshhold_4", temp, 0) )
		{
		a = atoi(temp);
		if(get_setup("apertur_level_4", temp, 0) )
			{
			b = atoi(temp);
			apcor_values[3] = ( (a & 0xf) << 4) + (b & 0xf); 		
			}
		}
	/* end apertur_correction */



	} /* end if setup file_present_flag */

/*	XXXXXX*/

/*
open changes parameter file. if this fails, alert user and use defaults
*/
sprintf(temp, "%s/change_parameters", frame_dir);
cfptr = fopen(temp, "r");
if(! cfptr)
	{
	fprintf(stderr,\
	"WARNING could not open file %s for read, using defaults\n",\
	temp);
	y_difference = 20;
	u_difference = 20;
	v_difference = 20;
	treshhold = 1000;
	}
else
	{
	fscanf(cfptr, "y_difference=%d\n", &y_difference);
	fscanf(cfptr, "u_difference=%d\n", &u_difference);
	fscanf(cfptr, "v_difference=%d\n", &v_difference);
	fscanf(cfptr, "treshhold=%d\n", &treshhold);
	fclose(cfptr);
	}

/* command line overrules whatever was read from setup */

if(cmd_line_internet_address)
	{
	if(internet_address) free(internet_address);
	internet_address = cmd_line_internet_address;
	}

if(cmd_line_internet_password)
	{
	if(remote_password) free(remote_password);
	remote_password = cmd_line_internet_password;
	}

/* ZZZZ */
if(cmd_line_client_flag)
	{
/*
	if(! cmd_line_internet_address)
		{
		fprintf(stderr, "IP address or name? ");
		fscanf(stdin, "%500s", temp);
		fprintf(stderr, "\n");
		}
	internet_address = strsave(temp);
	if(! internet_address)
		{
		fprintf(stderr,\
		"Cannot allocate space for internet address.\n");
		exit(1);
		}
	set_setup("internet_address", internet_address);
*/

	if(! cmd_line_internet_password)
		{
		fprintf(stderr, "password for this server? ");
		fscanf(stdin, "%500s", temp);
		remote_password = strsave(temp);
		if(! remote_password)
			{
			fprintf(stderr,\
			"Cannot allocate space for remote password\n");
			exit(1);
			}
		}
	
	if(! read_from_server(\
	argc, argv, internet_address, internet_port, remote_password) )
		{
		fprintf(stderr, "read from server failed\n");
		exit(1);
		}
	exit(1);
	}/* end if cmd_line_client_flag */

while(1)
 	{
	fprintf(stderr,\
	"\n\n   ======   mcam-%s  by Panteltje on LPT%d   =====\n\n",\
	VERSION, gPortNumber);
		
	CPIA_ModifyCameraStatus(2, ~(XXX_COM_FLAG | CPIA_FLAG), 0);

	/* set frames per second */
	if(! set_sensor_frames_per_second(sensor_frames_per_second, &fb) )
		{
		fprintf(stderr, "Could not set sensor frames per second\n");

		exit(1);
		}

	/*
	set apertur as recommended in developer.pdf
	(affect sharpness and noise)
	*/
	CPIA_SetApcor(\
	apcor_values[0], apcor_values[1], apcor_values[2], apcor_values[3]);

	rtn =\
	CPIA_GetCameraStatus(\
	&sys, &vp, &stream, &fatal, &cmd, NULL, &vp_stat, NULL ); 
	if ( !rtn )
		{
		fprintf(stderr, "Unable to GetCameraStatus\n" );
		}
	else /* have camera status */
		{
		fprintf(stderr,\
		"System=0x%02x  Cmd=0x%02x  Fatal=0x%02x\n",\
		sys, cmd, fatal );

 		if ( sys == HI_POWER_STATE )
 			{
			fprintf(stderr,\
			"VPCtrl=0x%02x  VPStat=0x%02x  Stream=0x%02x",\
			vp, vp_stat, stream );
			
			rtn =\
			CPIA_ReadVPRegs(\
			56, 57, 0, 0, &cam_status, &cam_setup0, NULL, NULL );
			if ( rtn )
				{
				fprintf(stderr,\
				"  CamStat=0x%02x   CamSetup0=0x%02x",\
				cam_status, cam_setup0 );
				}
			}/* end if hi power state */
		fprintf(stderr,"\n");
		}/* end else have camera status */

	/* get current exposure */
	CPIA_GetExposure(\
	(BYTE *)&current_gain,\
	(BYTE *)&current_fine_exposure,\
	(BYTE *)&current_coarse_exposure_low,\
	(BYTE *)&current_coarse_exposure_high,\
	(BYTE *)&red_compensation,\
	(BYTE *)&green_1_compensation, (BYTE *)&green_2_compensation,\
	(BYTE *)&blue_compensation);

	current_fine_exposure &= 0xff;

	/* bug fix: why is not the conversion done correctly? */
	current_gain &= 0xff;
	current_coarse_exposure_high &= 0xff;

	current_coarse_exposure =\
	(256 * current_coarse_exposure_high) + current_coarse_exposure_low;

	/* get current color gains */
	CPIA_GetColourBalance(\
	(BYTE *)&current_red_gain, (BYTE *)&current_green_gain,\
	(BYTE *)&current_blue_gain);

	/* bug fix: conversion problem? see above */
	current_red_gain &= 0xff;
	current_green_gain &= 0xff;
	current_blue_gain &= 0xff;

	 /* get brightness, contrast, saturation */
	CPIA_GetColourParams(\
	(BYTE *)&brightness, (BYTE *)&contrast, (BYTE *)&saturation );

	/* start menu */
	fprintf(stderr, "            menu\n");

	fprintf(stderr, "ESC quit\n");	

	if(man_exposure_flag)
		{
		fprintf(stderr, \
		"0 single frame grab\t\tA auto exposure on fine=%d coarse=%d <--.\n",\
		current_fine_exposure, current_coarse_exposure);
		
		fprintf(stderr, \
		"1 get Version.\t\t\ta auto exposure off.\n");
		}
	else
		{
		fprintf(stderr, \
		"0 single frame grab\t\tA auto exposure on.\n");

		fprintf(stderr, \
		"1 get version.\t\t\ta auto exposure off fine=%d coarse=%d <--.\n",\
		current_fine_exposure, current_coarse_exposure);
		}

	fprintf(stderr,  "2 sys. test (disabled EEPROM!).\tb brightness %d.\n",\
	brightness);

	fprintf(stderr,  "3 goto Suspend.\t\t\tc contrast (reverse gamma) %d.\n",\
	contrast);

	fprintf(stderr,  "4 goto LoPower (disabled!).\td set frame dir (%s).\n",
	frame_dir);

	if(compression_flag)
		{
		fprintf(stderr,  "5 goto HiPower.\t\t\tE enable compression of image uploads <--.\n");
		fprintf(stderr,  "6 goto Pass-Thru.\t\te disable compression of image uploads.\n");
		}
	else
		{
		fprintf(stderr,  "5 goto HiPower.\t\t\tE enable compression of image uploads.\n");
		fprintf(stderr,  "6 goto Pass-Thru.\t\te disable compression of image uploads <--.\n");
		}

	if(save_frames_flag)
		{
		fprintf(stderr, \
"7 wake from Pass-Thru.\t\tF save frames on start=%d delay=%d <--.\n",\
		start_frame, frame_delay);
		fprintf(stderr,  "8 force reset.\t\t\tf save frames off.\n");
		}
	else
		{
		fprintf(stderr, \
"7 wake from Pass-Thru.\t\tF save frames on delay=%d(always active).\n",\
		frame_delay);
		fprintf(stderr,  "8 force Reset.\t\t\tf save frames off <--.\n");
		}

	fprintf(stderr,  "9 continuous Grab/Upload.\tg set gain ceiling=%d.\n",\
	gain_ceiling);

	fprintf(stderr, "I internet connect to IP address (other mcam) %s.\n",\
	internet_address);

	fprintf(stderr, "i set internet port %d.\n", internet_port);

	if(color_depth_mode == 0)/* auto */
		{
		fprintf(stderr, \
		"J set color depth=%d (auto).\t",
		color_depth);
		}
	else
		{
		fprintf(stderr, \
		"J set color depth=%d (man).\t",\
		color_depth);
		}
	
	if(coding_mode == NO_CODING)
		{
		fprintf(stderr, "j coding sub menu (coding is off).\n");
		}
	else if(coding_mode == JPEG_CODING)
		{
		fprintf(stderr, "j coding sub menu (coding is jpeg).\n");
		}
	else if(coding_mode == MPEG2_CODING)
		{
		fprintf(stderr, "j coding sub menu (coding is mpeg).\n");
		}
	else if(coding_mode == WAVELET_CODING)
		{
		fprintf(stderr, "j coding sub menu (coding is wavelet).\n");
		}

	if(selected_font == S14_FONT)
		{
		fprintf(stderr, \
		"K character set no umlauts <--.\tk character set with umlauts.\n");
		}
	else if(selected_font == TXT_FONT)
		{
		fprintf(stderr, \
		"K character set no umlauts.\tk character set with umlauts. <--\n");
		}

	fprintf(stderr, \
	"L load and display frames sequentially movie replay (xbuf format).\n");
		
	fprintf(stderr, \
	"l load and display user specified frame number (xbuf format).\n");

	if(strcmp(server_control_password, "no") == 0)
		{
		fprintf(stderr, \
		"M accept IP connections.\tm set server control password (no).\n");
		}
	else
		{
		fprintf(stderr, \
		"M accept IP connections.\tm set server control password.\n");
		}

	fprintf(stderr, \
	"n toggle ccd divider contents (extra long exposure)\n");

	fprintf(stderr, \
	"N change frame rate (extra long exposure)\n");

	if(output_to_frame_flag)
		{
		fprintf(stderr, \
		"O output to framedir/mcam.ppm with handshake toggle (now on).\n");
		}
	else
		{
		fprintf(stderr, \
		"O output to framedir/mcam.ppm with handshake toggle (now off).\n");
		}

	if(picture_format == 1) /* big picture */
		{
		if(save_frames_format == 0)
			{
			fprintf(stderr, \
			"P big picture 352 x 288 <--.\tH save frames in format xbuf <--, ppm, yuv.\n");
			fprintf(stderr, \
			"p small picture 176 x 144.\n");
			}
		else if(save_frames_format == 1)
			{
			fprintf(stderr, \
			"P big picture 352 x 288 <--.\tH save frames in format xbuf, ppm <--, yuv.\n");
			fprintf(stderr, \
			"p small picture 176 x 144.\n");
			}
		else if(save_frames_format == 2)
			{
			fprintf(stderr, \
			"P big picture 352 x 288 <--.\tH frames in format xbuf, ppm, yuv <--.\n");
			fprintf(stderr, \
			"p small picture 176 x 144.\n");
			}
		}
	else /* small picture */
		{
		if(save_frames_format == 0)
			{
			fprintf(stderr, \
			"P big picture 352 x 288.\tH save frames in format xbuf <--, ppm, yuv.\n");
			fprintf(stderr, \
			"p small picture 176 x 144 <--.\n");
			}
		else if(save_frames_format == 1)
			{
			fprintf(stderr, \
			"P big picture 352 x 288.\tH save frames in format xbuf, ppm <--, yuv.\n");
			fprintf(stderr, \
			"p small picture 176 x 144 <--.\n");
			}
		else if(save_frames_format == 2)
			{
			fprintf(stderr, \
			"P big picture 352 x 288.\tH frames in format xbuf, ppm, yuv <--.\n");
			fprintf(stderr, \
			"p small picture 176 x 144 <--.\n");
			}
		}

	fprintf(stderr, "Q jpeg, mpeg, wavelet coding quality=%d\n", coding_quality);

	fprintf(stderr, "R exposure fine=%d coarse=%u gain=%d.\n",\
	current_fine_exposure, current_coarse_exposure, current_gain);

	fprintf(stderr, "r compensation r=%d g1=%d g2=%d b=%d.\n",\
	red_compensation, green_1_compensation, green_2_compensation,\
	blue_compensation); 

	if(strcmp(server_password, "no") == 0)
		{
		fprintf(stderr,  "S set server password (%s).\ts saturation %d.\n",\
		server_password, saturation);
		}
	else
		{
		fprintf(stderr,  "S set server password.\t\ts saturation %d.\n",\
		saturation);
		}

	fprintf(stderr, "T show time in picture toggle ");
	if(show_time_flag) fprintf(stderr, "now on\n");
	else fprintf(stderr, "now off\n");

	fprintf(stderr, "t time position now ");
	if (clock_position == TOP_LEFT) fprintf(stderr, "top left");
	else if(clock_position == TOP_MIDDLE) fprintf(stderr, "top middle");
	else if(clock_position == TOP_RIGHT) fprintf(stderr, "top right");
	else if(clock_position == BOTTOM_LEFT) fprintf(stderr, "bottom left");
	else if(clock_position == BOTTOM_MIDDLE) fprintf(stderr, "bottom middle");
	else if(clock_position == BOTTOM_RIGHT) fprintf(stderr, "bottom right");
	fprintf(stderr, "\n");
		
	if(special_effects_flag)
		{
		fprintf(stderr,
		"U special effects toggle (framedir/effects.dat) (now on)\n"); 
		}
	else
		{
		fprintf(stderr,
		"U special effects toggle (framedir/effects.dat) (now off)\n"); 
		}

	fprintf(stderr, "u time in UTC or local toggle ");
	if(use_utc_flag) fprintf(stderr, "now UTC\n");
	else fprintf(stderr, "now local\n");

	if(store_only_on_change_flag)
		{
		fprintf(stderr, "V store only if picture changes toggle (now on) <--.\n");
		}
	else
		{
		fprintf(stderr, "V store only if picture changes toggle (now off).\n");
		}

	fprintf(stderr, \
	"W apertur correction: treshold1 level1  t2 l2  t3 l3  t4 l4 (0-15).\n");
	fprintf(stderr, "\tcurrent apertur: %d %d  %d %d  %d %d  %d %d\n",\
	(apcor_values[0] & 0xf0) >> 4, apcor_values[0] & 0xf,\
	(apcor_values[1] & 0xf0) >> 4, apcor_values[1] & 0xf,\
	(apcor_values[2] & 0xf0) >> 4, apcor_values[2] & 0xf,\
	(apcor_values[3] & 0xf0) >> 4, apcor_values[3] & 0xf);

	fprintf(stderr, "w set apertur correction default (2 12  2 10  4 13  4 10)\n");

	fprintf(stderr, "X save current picture as .ppm, filename asked for.\n");

	if(color_balance_mode == 0)/* off */
		{
		fprintf(stderr, \
"Y auto colorb. r=%d g=%d b=%d.\ty man colorb. r=%d g=%d b=%d <--.\n",\
		current_red_gain, current_green_gain, current_blue_gain,\
		red_gain, green_gain, blue_gain);
		}
	else /* 2 = on, 0 is power up is default on */
		{ 
		fprintf(stderr, \
"Y auto colorb. r=%d g=%d b=%d <--.\ty man colorb. r=%d g=%d b=%d.\n",\
		current_red_gain, current_green_gain, current_blue_gain,\
		red_gain, green_gain, blue_gain);
		}

	fprintf(stderr, "Z set manual color balance values.\n");
	fprintf(stderr, "z set sensor frames per second (%d)\n",\
	sensor_frames_per_second);

	if (ccddivide)
		{
		fprintf(stderr,\
		"Warning you used option 'n' cmos sensor divider is set to 16!\n");
		}

	/* end menu */
	
	/* important messages (that should not scroll of the screen) */
	if(output_to_frame_flag)
		{
		fprintf(stderr,\
"\nHANDSHAKE OUTPUT MODE 'O' ACTIVATED,\n\
mcam will now wait for %s/mcam.ppm.ready to be deleted,\n\
before it will read the next frame into %s/mcam.ppm,\n\
and creates a new %s/mcam.ppm.ready.\n\
ctrl C aborts\n",\
		frame_dir, frame_dir, frame_dir); 
		}/* end if output_to_frame_flag */
	
	/* end important messages */

	if(cmd_line_server_flag)/* start server */
		{
		ch = 'M';
		}
	else if(output_to_frame_flag)/* handshake mcam.ppm.ready */
		{
		ch = '9';
		}
	else /* show menu */
		{
		/* read a user key */	
		system("stty raw");
		ch = getc(stdin);
		system("stty sane");
		}
	fprintf(stderr, "\n");

	switch( ch )
		{
		// added by NDJ
		case '0': /* single frame grab */
			fprintf(stderr, "single frame grab\n");

			CPIA_SerialStart();	// setting ccd divider to normal
			CPIA_SerialWrite(0x20);
			CPIA_SerialWrite(0x25);
			CPIA_SerialWrite(0x00);
			CPIA_SerialStop();

			/* get a frame from the camera into gStreamBuffer */
			if(! get_a_frame((int *)&actual_bytes, coding_mode) )
				{
				fprintf(stderr, "Get_a_frame failed\n");
				}

			/* insert display here */
			if(! display_a_frame(\
			argc, argv, actual_bytes, coding_mode, compression_flag) )
				{
				fprintf(stderr, "Display_a_frame failed\n");
				}

			CPIA_SetSensorFPS(framerate, baseclk); // restoring clk speed
			if (ccddivide)
				{
				CPIA_SerialStart();
				CPIA_SerialWrite(0x20);
				CPIA_SerialWrite(0x25);
				CPIA_SerialWrite(0x03);	// divider to 16
				CPIA_SerialStop();
				}
			else
				{
				CPIA_SerialStart();
				CPIA_SerialWrite(0x20);
				CPIA_SerialWrite(0x25);
				CPIA_SerialWrite(0x00);	// divider to 2
				CPIA_SerialStop();
				}
			break;
		// added by NDJ
		case 'n': /* toggle ccd divider contents */
			ccddivide = 1 - ccddivide;
			if (ccddivide)
				{
				fprintf(stderr,\
				"changing cmos sensor divider to 16\n");
				CPIA_SerialStart();
				CPIA_SerialWrite(0x20);
				CPIA_SerialWrite(0x25);
				CPIA_SerialWrite(0x03);	// divider to 16
				CPIA_SerialStop();
				}
			else
				{
				fprintf(stderr,\
				"changing cmos sensor divider to 2 (original)\n");
				CPIA_SerialStart();
				CPIA_SerialWrite(0x20);
				CPIA_SerialWrite(0x25);
				CPIA_SerialWrite(0x00);	// divider to 2
				CPIA_SerialStop();
				}

//			grabmode = 1 - grabmode;
// grabmode getest en voorlopig zinloos gevonden
//			if (grabmode)
//				{
//				fprintf(stderr,\
//				"Setting grab mode to continuous\n");
//				}
//			else
//				{
//				fprintf(stderr,\
//				"Setting grab mode to single frame grab\n");
//				}
//			CPIA_SetGrabMode(grabmode);
//			wait_key();

			break;
		// added by NDJ
		case 'N':  /* change frame rate */
			fprintf(stderr,\
			"Enter baserate (currently %d), 1=30fps, 0=25fps? ",\
			baseclk);
			fscanf(stdin, "%d", &baseclk);
			fprintf(stderr,\
			"Enter divisor (currently %d) 0=div1 1=div2 2=div4 3=div8 ",\
			framerate);
			fscanf(stdin, "%d", &framerate);
			CPIA_SetSensorFPS(framerate, baseclk);
			wait_key();
			break;
	   	case 27:
			CPIA_CmdFini();
			CLINK_Fini();
			exit(0);
	   		break;
		case '1':
			rtn =\
			CPIA_GetVersion( &gFW_Ver, &gFW_Rev, &gVC_Ver, &gVC_Rev );
	 		if ( rtn )
	 			{
				fprintf(stderr,  "\nFirmware = v%d-%02d        ",\
				gFW_Ver, gFW_Rev );	
				fprintf(stderr,  "Video Compressor = v%d-%02d\n\n",\
				gVC_Ver, gVC_Rev );	
				}
			wait_key();
			break;
		case '2': /* System Test, this will write to EEPROM, disabled */
/*
			rtn = DoSystemTest();	
			if ( !rtn ) fprintf(stderr,  "\nSystem test = FAILED\n\n" );
			else fprintf(stderr,  "\nSystem test = PASSED\n\n" ); 
			wait_key();
*/
			break;
		case '3':
			rtn = CPIA_GotoSuspend();
			if ( !rtn )
				{
				fprintf(stderr,  "Failed to send GotoSuspend command ...\n\n" );
				}
			else fprintf(stderr,  "\nGotoSuspend command sent OK ...\n\n" ); 
			wait_key();
			break;
//		case '4':
			/*
			Option 4 (goto LoPower) disabled.
			Choice of this option results in hanging port since at least one
			of the commands in the main loop (SetApcor, Get Exposure ...)
			seems to require HiPower for the CPIA1_00 to be correctly
			transfered.
			*/

//			CPIA_GotoLoPower();
//			break;
		case '5':
			CPIA_GotoHiPower();
			break;
		case '6':
			CPIA_WriteVCReg( 0, 0x7F, 0x80 );
			CPIA_GotoPassThrough();
			break;
		case '7':
			CLINK_SendWakeup();
			break;
		case '8':
			CLINK_ForceWatchdogReset();
			break;
		case '9':
			CPIA_GotoHiPower();
			fprintf(stderr,\
			"ENTER for menu, control Z to freeze, fg to continue\n");
			if(save_frames_flag)
				{
				fprintf(stderr, "Saving frames\n");
				}

 			frames_displayed = 0;
 			frame_count = start_frame;
 			start_time = time(0);
 			while(1)/* repeat grab */
 				{
				if(console_status() )
					{
					end_time = time(0);
					fprintf(stderr, "User abort grab\n");
					elapsed_seconds = end_time - start_time;
										
					frames_per_second =\
					(float)frames_displayed / (float)elapsed_seconds;
					fprintf(stderr, "Frames per second was %.2f\n",\
					frames_per_second);
					if(frame_delay != 0)
						{
						fprintf(stderr,\
						"Could not calculate true frames per second\n\
frame delay was set to %d\n",\
						frame_delay);
						}
					break;
					}

				/* get a frame from the camera into gStreamBuffer */
				if(! get_a_frame((int *)&actual_bytes, coding_mode) )
					{
					break;
					}

				/* insert display here */
					
				if(! display_a_frame(\
				argc, argv, actual_bytes, coding_mode, compression_flag) )
					{
					fprintf(stderr, "Display_a_frame failed\n");
					break;
					}
	
				frames_displayed++;

				}/* end while grab pictures */

			wait_key();			
/*			CPIA_GotoLoPower();*/
			break;
		case 'A': /* AEC on */
			man_exposure_flag = 1;
			set_setup("auto_exposure_control", "on");
			CPIA_SetExposure(0, 2, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
	   		break;
	   	case 'a': /* AEC off */
			man_exposure_flag = 0;
			set_setup("auto_exposure_control", "off");
			CPIA_SetExposure(0, 3, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
			break;	   	
	   	case 'b':/* brightness */
			fprintf(stderr,\
			"AEC target exposure: brightness=%d brightness(0-100)? \n",\
			brightness);
			fscanf(stdin, "%d", &brightness);
			CPIA_SetColourParams( brightness, contrast, saturation );

			sprintf(temp, "%d", brightness);
			set_setup("brightness", temp);
			break;
	   	case 'c':/* contrast */
			fprintf(stderr,\
			"Reverse gamma: contrast=%d contrast(0-100)? \n",\
			contrast);
			fscanf(stdin, "%d", &contrast);
			CPIA_SetColourParams( brightness, contrast, saturation );

			sprintf(temp, "%d", contrast);
			set_setup("contrast", temp);
			break;
		case 'd':/* set frame directory */
			fprintf(stderr, "Frame directory (frame_dir in ~/.mcamrc)? ");
			fscanf(stdin, "%500s", temp);
			if(frame_dir) free(frame_dir);
			frame_dir = strsave(temp);
			if(! frame_dir)
				{
				fprintf(stderr, "Cannot allocate space for frame_dir\n");
				}
			set_setup("frame_dir", frame_dir);
			break;
	   	case 'E':/* enable auto compression before upload */
	   		compression_flag = 1;
	   		set_setup("compression", "on");
	   		CPIA_SetCompression(1, 0);
	   		/* 0 off  1 auto  2 man, 0 no decimation  1 decimation  */
	   		break;
		case 'e':/* disable auto compression before upload */		   	
	   		/* 0 off  1 auto  2 man, 0 no decimation  1 decimation  */
	   		compression_flag = 0;
	   		set_setup("compression", "off");
	   		CPIA_SetCompression(0, 0);
	   		break;
  		case 'F':/* save frames on */
  			save_frames_flag = 1;
			set_setup("save_frames", "on");
			
			fprintf(stderr, "Start frame? ");
			fscanf(stdin, "%d", &start_frame);
			sprintf(temp, "%d", start_frame);
			set_setup("save_frames_start_frame", temp);

			fprintf(stderr, "\nDelay between frames (milli seconds)? ");
			fscanf(stdin, "%d", &frame_delay);
			sprintf(temp, "%d", frame_delay);
			set_setup("save_frames_frame_delay", temp);
					
			fprintf(stderr, "\n");
			break;
  		case 'f':/* save frames off */
  			save_frames_flag = 0;
  			set_setup("save_frames", "off");
  			break;
		case 'g': /* gain */
			/* read current */
			CPIA_GetExposure(\
			(BYTE *)&current_gain,\
			(BYTE *)&current_fine_exposure,\
			(BYTE *)&current_coarse_exposure_low,\
			(BYTE *)&current_coarse_exposure_high,\
			(BYTE *)&red_compensation,\
			(BYTE *)&green_1_compensation, (BYTE *)&green_2_compensation,\
			(BYTE *)&blue_compensation);
			
			/* perhaps bug fix: see above for 'bugfix' */
			current_gain &= 0xff;
			current_coarse_exposure_low &= 0xff;
			current_coarse_exposure_high &= 0xff;

			current_fine_exposure &= 0xff;

			red_compensation &= 0xff;
			green_1_compensation &= 0xff;
			green_2_compensation &= 0xff;
			blue_compensation &= 0xff;

			current_coarse_exposure =\
			(256 * current_coarse_exposure_high) +\
			current_coarse_exposure_low;
			
			/* ask user for new value */
			fprintf(stderr,\
			"gain_ceiling=%d (1(x1) 2(x2) 3(x4) 4(x8)? ",\
			gain_ceiling);
			fscanf(stdin, "%d", &gain_ceiling);
			fprintf(stderr, "\n");
			
			CPIA_SetExposure(gain_ceiling, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0);
/*
			gain, fine_exposure, coarse_exposure_low, coarse_exposure_high,\
			red_compensation, green_1_compensation, green_2_compensation,\
			blue_compensation);
*/
			sprintf(temp, "%d", gain_ceiling);
			set_setup("gain_ceiling", temp);
			break;
  		case 'H': /* save frames format */
#ifdef USE_X
			fprintf(stderr, "save format? (xbuffer, ppm, yuv)\n");
#else
			fprintf(stderr, "save format? (ppm, yuv)\n");
#endif
			fscanf(stdin, "%500s", temp);
			if(strncmp(temp, "yuv", 3) == 0)
				{
	  			save_frames_format = 2;
  				set_setup("save_frames_format", "yuv");
				}
#ifdef USE_X
			else if(strncmp(temp, "ppm", 3) == 0)
				{
	  			save_frames_format = 1;
  				set_setup("save_frames_format", ".ppm");
				}
#endif
			else 
				{
	  			save_frames_format = 0;
  				set_setup("save_frames_format", "xbuffer");
				}
  			break;
// 		case 'h': /* save frames in .ppm format */
//  			save_frames_format = 1;
//	 			set_setup("save_frames_format", ".ppm");
// 			break;
  		case 'I': /* connect to internet IP address */
			fprintf(stderr, "IP address or name? ");
			fscanf(stdin, "%500s", temp);
			fprintf(stderr, "\n");

			internet_address = strsave(temp);
			if(! internet_address)
				{
				fprintf(stderr,\
				"Cannot allocate space for internet address.\n");
				exit(1);
				}
			set_setup("internet_address", internet_address);
			
			fprintf(stderr, "password for this server? ");
			fscanf(stdin, "%500s", temp);
			remote_password = strsave(temp);
			if(! remote_password)
				{
				fprintf(stderr,\
				"Cannot allocate space for remote_password\n");
				exit(1);
				}
			fprintf(stderr, "\n");
	
			if(! read_from_server(\
			argc, argv, internet_address, internet_port, remote_password) )
				{
				fprintf(stderr, "ENTER for menu\n");
				wait_key();
				wait_key(); /* why 2 needed? */
				}
			break;
  		case 'i': /* internet port */
  			fprintf(stderr, "Internet port? ");
  			fscanf(stdin, "%d", &internet_port);
  			sprintf(temp, "%d", internet_port);
  			fprintf(stderr, "\n");
  			
  			set_setup("internet_port", temp);
  			break;
#ifdef USE_X
  		case 'J': /* set color_depth (auto / man + value) */
			fprintf(stderr, "Color depth mode? (m for man / a for auto) ");
			fscanf(stdin, "%s", temp);
  			if( toupper(temp[0]) == 'M') color_depth_mode = 1; /* manual */
  			else color_depth_mode = 0;/* auto */
			fprintf(stderr, "\n");
			sprintf(temp, "%d", color_depth_mode);
  			set_setup("color_depth_mode", temp);
  			
  			/* ask the value if manual */
  			if(color_depth_mode == 1)
  				{
	  			fprintf(stderr, "Color depth? ");
  				fscanf(stdin, "%d", &color_depth);
  				sprintf(temp, "%d", color_depth);
	  			fprintf(stderr, "\n");
  			
  				sprintf(temp, "%d", color_depth);
  				set_setup("color_depth", temp);
  				}

  			break;
#endif /* USE_X */
		case 'j': /* select between coding off, jpeg, mpeg, wavelet */
			fprintf(stderr, "\t\tcoding submenu.\n");
			fprintf(stderr,\
			"\tjpeg coding requires libjpeg.\n");
			fprintf(stderr,\
			"\tmpeg coding requires modified Berkely mpeg2 codec.\n");
			fprintf(stderr,\
			"\twavelet coding requires epic codec.\n");
			fprintf(stderr,\
			"\tset coding quality with 'Q' in main menu.\n");
			fprintf(stderr,\
			"\tA lower Q (say 25) gives better compression.\n");

			while(1)
				{
				if(coding_mode == NO_CODING)
					{
					fprintf(stderr,\
					"0 coding off <--\n1 jpeg \n2 mpeg2 \n3 wavelet\n");
					}
				else if(coding_mode == JPEG_CODING)
					{
					fprintf(stderr,\
					"0 coding off\n1 jpeg <--\n2 mpeg2 \n3wavelet\n");
					}
				else if(coding_mode == MPEG2_CODING)
					{
					fprintf(stderr,\
					"0 coding off\n1 jpeg \n2 mpeg2 <-- \n3 wavelet\n");
					}
				else if(coding_mode == WAVELET_CODING)
					{
					fprintf(stderr,\
					"0 coding off\n1 jpeg \n2 mpeg2 \n3wavelet <--\n");
					}
				fprintf(stderr, "enter choice\n");
			
				fscanf(stdin, "%d", &a);

				if(a == 0)
					{
					set_setup("coding", "off");
					coding_mode = NO_CODING;
					break;
					}
				else if(a == 1)
					{
					coding_mode = JPEG_CODING;
					set_setup("coding", "jpeg");
					break;
					}
				else if(a == 2)
					{
					coding_mode = MPEG2_CODING;
					set_setup("coding", "mpeg");
					break;
					}
				else if(a == 3)
					{
					coding_mode = WAVELET_CODING;
					set_setup("coding", "wavelet");
					break;
					}
				}/* while input */
			break;
  		case 'M': /* accept IP connection */
			/*
			ignore SIGPIPE for clients closing connection, or else
			mcam server would exit.
			*/ 
  			signal (SIGPIPE, SIG_IGN);
  			
  			CPIA_GotoHiPower();

			/* prevent hanging in server mode */
			output_to_frame_flag = 0;

			if(! accept_connections(argc, argv, internet_port, coding_mode) )
				{
				fprintf(stderr, "ENTER for menu\n");
				wait_key();
				cmd_line_server_flag = 0;
				}

			/* restore default signal handling for SIGPIPE */
			signal (SIGPIPE, SIG_DFL);	
  			break;
  		case 'm': /* set server control password */
			fprintf(stderr, "server control password? ");
			fscanf(stdin, "%15s", temp);

			free(server_control_password);
			server_control_password = strsave(temp);
			if(! server_control_password)
				{
				fprintf(stderr,\
				"Cannot allocate space for server_control_password\n");
				exit(1);
				}
			fprintf(stderr, "\n");
  			set_setup("server_control_password", temp);
  			break;
		case 'K':/* character set no umlauts (s14font) */
			selected_font = S14_FONT;
			set_setup("font", "s14font");
			break;
		case 'k': /* character set with umlauts (txtfont) */
			selected_font = TXT_FONT;
			set_setup("font", "txtfont");
			break;	
#ifdef USE_X
  		case 'L':/* load frames sequentially from 0 (replay) */
			fprintf(stderr, "Start frame number? ");
			fscanf(stdin, "%d", &i);
			
			fprintf(stderr, "\ndelay in milliseconds? ");
			fscanf(stdin, "%d", &milliseconds_delay);
			fprintf(stderr, "\nENTER to abort (or ctrl Z)\n");

			while(1)
				{
  				if(console_status() ) break;

  				fprintf(stderr, "frame %d\n", i);

				sprintf(temp, "%s/frame.%d", frame_dir, i);
				if(! read_and_display_frame(temp, argc, argv) ) break; 
  				i++;
  				usleep(milliseconds_delay * 1000);
  				}

  			break;
  		case 'l': /* display selected frame */
			fprintf(stderr, "Frame number? ");
			fscanf(stdin, "%d", &a);
			fprintf(stderr, "\n");

			sprintf(temp, "%s/frame.%d", frame_dir, a);
			read_and_display_frame(temp, argc, argv);

			wait_key();			
  			break;
#endif /* USE_X */
		case 'O': /* output to frame with handshake */
			output_to_frame_flag = 1 - output_to_frame_flag;
			/*
			if(output_to_frame_flag) set_setup("output_to_frame", "yes");
			else set_setup("output_to_frame", "no");			
			*/
			break;
		case 'P': /* big picture */
			picture_format = 1;
			set_setup("size", "big");
			CPIA_SetFormat(picture_format, 1, 0);
			/*
			arg 1 VideoSize: 0 = QCIF (176x144) 1 = CIF(352x288)
			arg 2 SubSample: 0 = 4:2:0 1 = 4:2:2
			arg 3 YUVOrder: 0 = YUYV 1 = UYVY
			*/

			break;
		case 'p': /* small picture */
			picture_format = 0;
			set_setup("size", "small");
			CPIA_SetFormat(picture_format, 1, 0);
			/*
			arg 1 VideoSize: 0 = QCIF (176x144) 1 = CIF(352x288)
			arg 2 SubSample: 0 = 4:2:0 1 = 4:2:2
			arg 3 YUVOrder: 0 = YUYV 1 = UYVY
			*/

			break;
		case 'Q': /* coding quality default 75 useful 5 to 95 */
			fprintf(stderr,\
		"Current quality = %d quality? (useful is 20 to 80)?\n",\
		coding_quality);
			fscanf(stdin, "%d", &coding_quality);
			sprintf(temp, "%d", coding_quality);
			set_setup("coding_quality", temp);
			break;
		case 'R': /* set man exposure time and gain */
			/* bugs in CPIA chip, see developer.pdf page 16 */
			/* read current */
			CPIA_GetExposure(\
			(BYTE *)&current_gain,\
			(BYTE *)&current_fine_exposure,\
			(BYTE *)&current_coarse_exposure_low,\
			(BYTE *)&current_coarse_exposure_high,\
			(BYTE *)&red_compensation,\
			(BYTE *)&green_1_compensation, (BYTE *)&green_2_compensation,\
			(BYTE *)&blue_compensation);

			/* perhaps bug fix: see above for 'bugfix' */
			current_gain &= 0xff;
			current_coarse_exposure_low &= 0xff;
			current_coarse_exposure_high &= 0xff;

			current_fine_exposure &= 0xff;

			red_compensation &= 0xff;
			green_1_compensation &= 0xff;
			green_2_compensation &= 0xff;
			blue_compensation &= 0xff;

			current_coarse_exposure =\
			(256 * current_coarse_exposure_high) +\
			current_coarse_exposure_low;
			
			/* ask user for new value */
			fprintf(stderr, "current fine exposure=%d fine_exposure? ",\
			current_fine_exposure);
			fscanf(stdin, "%d", &fine_exposure);
			fprintf(stderr, "\n");
			
			sprintf(temp, "%d", fine_exposure);
			set_setup("fine_exposure", temp);

			fprintf(stderr,\
			"current coarse_exposure=%d coarse_exposure? ",\
			current_coarse_exposure);
			fscanf(stdin, "%d", &coarse_exposure);
			fprintf(stderr, "\n");

			coarse_exposure_low = coarse_exposure % 256;
			coarse_exposure_high = coarse_exposure / 256;

			sprintf(temp, "%d", coarse_exposure);
			set_setup("coarse_exposure", temp);
			
			fprintf(stderr,\
			"current gain=%d gain? ",\
			current_gain);
			fscanf(stdin, "%d", &gain);
			fprintf(stderr, "\n");

			sprintf(temp, "%d", gain);
			set_setup("gain", temp);
			
			CPIA_SetExposure(0, 1, 0, 0,\
			(BYTE)gain, (BYTE)fine_exposure,\
			(BYTE)coarse_exposure_low, (BYTE)coarse_exposure_high,\
			0, 0, 0, 0);

			wait_key();
			break;
		case 'r':/* compensation gains */
			/* bugs in CPIA chip, see developer.pdf page 16 */
			/* read current */
			CPIA_GetExposure(\
			(BYTE *)&current_gain,\
			(BYTE *)&current_fine_exposure,\
			(BYTE *)&current_coarse_exposure_low,\
			(BYTE *)&current_coarse_exposure_high,\
			(BYTE *)&red_compensation,\
			(BYTE *)&green_1_compensation, (BYTE *)&green_2_compensation,\
			(BYTE *)&blue_compensation);

			/* perhaps bug fix: see above for 'bugfix' */
			current_gain &= 0xff;
			current_coarse_exposure_low &= 0xff;
			current_coarse_exposure_high &= 0xff;

			current_fine_exposure &= 0xff;

			red_compensation &= 0xff;
			green_1_compensation &= 0xff;
			green_2_compensation &= 0xff;
			blue_compensation &= 0xff;

			current_coarse_exposure =\
			(256 * current_coarse_exposure_high) +\
			current_coarse_exposure_low;

			fprintf(stderr,\
			"current red_compensation=%d red_compensation? ",\
			red_compensation);
			fscanf(stdin, "%d", &red_compensation);
			fprintf(stderr, "\n");
			sprintf(temp, "%d", red_compensation);
			set_setup("red_compensation", temp);
			
			fprintf(stderr,\
			"current green_1_compensation=%d green_1_compensation? ",\
			green_1_compensation);
			fscanf(stdin, "%d", &green_1_compensation);
			fprintf(stderr, "\n");
			sprintf(temp, "%d", green_1_compensation);
			set_setup("green_1_compensation", temp);
			
			fprintf(stderr,\
			"current green_2_compensation=%d green_2_compensation? ",\
			green_2_compensation);
			fscanf(stdin, "%d", &green_2_compensation);
			fprintf(stderr, "\n");
			sprintf(temp, "%d", green_2_compensation);
			set_setup("green_2_compensation", temp);
			
			fprintf(stderr,\
			"current blue_compensation=%d blue_compensation? ",\
			blue_compensation);
			fscanf(stdin, "%d", &blue_compensation);
			fprintf(stderr, "\n");
			sprintf(temp, "%d", blue_compensation);
			set_setup("blue_compensation", temp);

			CPIA_SetExposure(0, 0, 1, 0,  0, 0 ,0, 0,\
			(BYTE)red_compensation,\
			(BYTE)green_1_compensation, (BYTE)green_2_compensation,\
			(BYTE)blue_compensation);

			wait_key();
			break;
  		case 'S': /* set server password for connect */
			fprintf(stderr, "server password? ");
			fscanf(stdin, "%15s", temp);

			free(server_password);
			server_password = strsave(temp);
			if(! server_password)
				{
				fprintf(stderr, "Cannot allocate space for server_password\n");
				exit(1);
				}
			fprintf(stderr, "\n");
  			set_setup("server_password", temp);
  			break;
  		case 's':/* saturation */
			fprintf(stderr,\
			"saturation=%d saturation(0-100)? \n",\
			saturation);
			fscanf(stdin, "%d", &saturation);
			CPIA_SetColourParams( brightness, contrast, saturation );

			sprintf(temp, "%d", saturation);
			set_setup("saturation", temp);
			break;
		case 'T':/* show time toggle */
			show_time_flag = 1 - show_time_flag;
			
			if(show_time_flag) set_setup("show_time", "yes");
			else set_setup("show_time", "no");
			break;
		case 't':/* clock position */
			fprintf(stderr, "\t\tClock position sub menu\n\n");
			fprintf(stderr, "1\ttop left\n");
			fprintf(stderr, "2\ttop middle\n");
			fprintf(stderr, "3\ttop right\n");
			fprintf(stderr, "4\tbottom left\n");
			fprintf(stderr, "5\tbottom middle\n");
			fprintf(stderr, "6\tbottom right\n");
			fprintf(stderr, "\nEnter choice\n");
			
			fscanf(stdin, "%s", temp);
			clock_position = atoi(temp);

			if(clock_position == 1) set_setup("clock_position", "top_left");
			if(clock_position == 2) set_setup("clock_position", "top_middle");
			if(clock_position == 3) set_setup("clock_position", "top_right");
			if(clock_position == 4) set_setup("clock_position", "bottom_left");
			if(clock_position == 5) set_setup("clock_position", "bottom_middle");
			if(clock_position == 6) set_setup("clock_position", "bottom_right");

			break;
		case 'U':/* special effects toggle */
			special_effects_flag = 1 - special_effects_flag;
			break;
		case 'u':/* time in UTC  toggle */
			use_utc_flag = 1 - use_utc_flag;
			
			if(use_utc_flag) set_setup("time", "utc");
			else set_setup("time", "local");
			break;
	   	case 'V':/* store on change toggle */
			store_only_on_change_flag = 1 - store_only_on_change_flag;
	   		if(store_only_on_change_flag)
	   			{
	   			set_setup("store_only_on_change", "yes");
	   			}
	   		else set_setup("store_only_on_change", "no");
	   		break;
		case 'W': /* apertur correction */
			fprintf(stderr, "Apertur correction:\n");
			for(i = 0; i < 4; i++)
				{
				fprintf(stderr, "treshold%d (0-15)? ", i + 1);
				fscanf(stdin, "%d", &a);

				fprintf(stderr, "level%d (0-15)? ", i + 1);
				fscanf(stdin, "%d", &b);

				apcor_values[i] = ( (a & 0xf) << 4) + (b & 0xf); 		
				fprintf(stderr, "\n");
				}

			sprintf(temp, "%d", (apcor_values[0] & 0xf0) >> 4);
			set_setup("apertur_treshhold_1", temp);
			sprintf(temp, "%d", apcor_values[0] & 0xf);
			set_setup("apertur_level_1", temp);

			sprintf(temp, "%d", (apcor_values[1] & 0xf0) >> 4);
			set_setup("apertur_treshhold_2", temp);
			sprintf(temp, "%d", apcor_values[1] & 0xf);
			set_setup("apertur_level_2", temp);

			sprintf(temp, "%d", (apcor_values[2] & 0xf0) >> 4);
			set_setup("apertur_treshhold_3", temp);
			sprintf(temp, "%d", apcor_values[2] & 0xf);
			set_setup("apertur_level_3", temp);

			sprintf(temp, "%d", (apcor_values[3] & 0xf0) >> 4);
			set_setup("apertur_treshhold_4", temp);
			sprintf(temp, "%d", apcor_values[3] & 0xf);
			set_setup("apertur_level_4", temp);

			break;
		case 'w': /* set apertur correction default */
			/*
			set apertur as recommended in developer.pdf
			(affect sharpness and noise)
			*/
			apcor_values[0] = 0x2c;
			apcor_values[1] = 0x2a;
			apcor_values[2] = 0x4d;
			apcor_values[3] = 0x4a;

			CPIA_SetApcor(\
			apcor_values[0], apcor_values[1], apcor_values[2], apcor_values[3]);

			sprintf(temp, "%d", (apcor_values[0] & 0xf0) >> 4);
			set_setup("apertur_treshhold_1", temp);
			sprintf(temp, "%d", apcor_values[0] & 0xf);
			set_setup("apertur_level_1", temp);

			sprintf(temp, "%d", (apcor_values[1] & 0xf0) >> 4);
			set_setup("apertur_treshhold_2", temp);
			sprintf(temp, "%d", apcor_values[1] & 0xf);
			set_setup("apertur_level_2", temp);

			sprintf(temp, "%d", (apcor_values[2] & 0xf0) >> 4);
			set_setup("apertur_treshhold_3", temp);
			sprintf(temp, "%d", apcor_values[2] & 0xf);
			set_setup("apertur_level_3", temp);

			sprintf(temp, "%d", (apcor_values[3] & 0xf0) >> 4);
			set_setup("apertur_treshhold_4", temp);
			sprintf(temp, "%d", apcor_values[3] & 0xf);
			set_setup("apertur_level_4", temp);

			break;

#ifdef USE_X
	   	case 'X': /* save current picture, filename asked for */
	   		/* check if picture present */
	   		if(! window_open_flag)
	   			{
	   			fprintf(stderr, "No picture present\n");
	   			wait_key();
	   			break;
	   			}
	   		
	   		/* ask user for path_filename */
	   		fprintf(stderr, "Save picture as? ");
	   		fscanf(stdin, "%1000s", path_filename);
			fprintf(stderr, "\n");
						
			/* append .ppm if not already there */
			a = strlen(path_filename);
			if(strcmp(path_filename + a - 4, ".ppm") != 0) 
				{
				strcat(path_filename, ".ppm");
				}
			fptr = fopen(path_filename, "w");
			if(!fptr)
				{
				fprintf(stderr,\
				"Save picture: cannot open file %s for write\n",\
				path_filename);
				wait_key();
				wait_key();/* why 2 needed? */
				strerror(errno);
				break;
				}
			
			if(picture_format == 1)
				{
				gstream_to_ppm(\
				352, 288, gStreamBuffer, actual_bytes, fptr,\
				compression_flag);
				}
			else
				{
				gstream_to_ppm(\
				176, 144, gStreamBuffer, actual_bytes, fptr,\
				compression_flag);
				}   		
	   		break;
#endif /*USE_X */
	   	case 'Y': /* set automatic color balance on */
			color_balance_mode = 1;

			/* set auto */
			CPIA_SetColourBalance(2, 0, 0, 0);

			set_setup("color_balance_mode", "auto");						
			break;
	   	case 'y': /* set manual color balance */
			color_balance_mode = 0;

			/* set manual */
			CPIA_SetColourBalance(3, 0, 0, 0);

			/* set values */
			CPIA_SetColourBalance(1, red_gain, green_gain, blue_gain);

			set_setup("color_balance_mode", "man");						
	   		break;
	   	case 'Z': /* set manual color balance submenu */
			/* show the current values */
			fprintf(stderr, "the current settings are\n\
			red_gain=%d green_gain=%d blue_gain=%d\n",\
			red_gain, green_gain, blue_gain);
			
	   		fprintf(stderr, "Manual color balance submenu\n");

	   		fprintf(stderr,"\nred_gain=%d red_gain (0-212)? ",\
	   		red_gain);
	   		fscanf(stdin, "%d", &red_gain);

	   		fprintf(stderr, "\ngreen_gain=%d green_gain(0-212)? ",\
	   		green_gain);
	   		fscanf(stdin, "%d", &green_gain);

	   		fprintf(stderr, "\nblue_gain=%d blue_gain(0-212)? ",\
	   		blue_gain);
	   		fscanf(stdin, "%d", &blue_gain);

			fprintf(stderr, "\n");   		

			/* set manual */
			CPIA_SetColourBalance(3, 0, 0, 0);

			/* use arguments */
			CPIA_SetColourBalance(1, red_gain, green_gain, blue_gain);

	   		/* manual mode */
	   		color_balance_mode = 0;

	   		set_setup("color_balance_mode", "man");

	   		sprintf(temp, "%d", red_gain);
	   		set_setup("red_gain", temp);

	   		sprintf(temp, "%d", green_gain);
	   		set_setup("green_gain", temp);

	   		sprintf(temp, "%d", blue_gain);
	   		set_setup("blue_gain", temp);
	   		break;
		case 'z': /* set frames per second */
			fprintf(stderr,\
"frames per second sub menu, this has effect on the maximum exposure time\n\
use (1-30), but this will be converted one of these:\n\
			30\n\
			25\n\
			15\n\
			12.5\n\
			7.5\n\
			6.25\n\
			3.75\n\
			3.125\n");

			fscanf(stdin, "%s", temp);
			a = atoi(temp);

			set_sensor_frames_per_second(a, &fb);
			fprintf(stderr,\
			"sensor frames per second set to=%.3f\n",\
			fb);

			sensor_frames_per_second = a;

	   		sprintf(temp, "%d", a);
	   		set_setup("sensor_frames_per_second", temp);

			break;
	   	}/* end switch */

	}/* end while read keys */

/* never here */
}/* end function main */


BOOL CheckForCPIA( BYTE PortNumber )
{
BOOL rtn;
WORD base;
BYTE type, cmd, fatal;
char deviceID[ 256 ];

/*
modified for CPIA1_00 chip version, with thanks to
Heinz-Jrgen Steinhoff <hjs@bph.ruhr-uni-bochum.de> for the fix.
*/

if ( !CLINK_DetectPort( PortNumber, &base, &type ) )
	{
	fprintf(stderr,  "\nmcam: LPT%d does not exist\n", PortNumber );
	return FALSE;
	}
else
	{
	fprintf(stderr,  "\nLPT%d at 0x%x -- ", PortNumber, base );
	if ( type & ECP_PORT_TYPE ) fprintf(stderr,  "ECP / " );
	if ( type & COMPAT_PORT_TYPE ) fprintf(stderr,  "4 Bit\n" );
	fprintf(stderr,  "\n" );
	}

if ( !CLINK_SelectPort( PortNumber ) )
	{
	fprintf(stderr,  "mcam: SelectPort() failed\n" );
	return FALSE;
	}

if ( !CLINK_SetPortInfo( type, 800 ) )
	{
	fprintf(stderr,  "mcam: GetPortInfo() failed\n" );
	return FALSE;
	}

if ( !CLINK_SendWakeup() )
	{
	fprintf(stderr,  "mcam: SendWakeup() failed\n" );
	return FALSE;
	}

if ( !CLINK_Get1284DeviceID( deviceID, 256 ) )
	{
	fprintf(stderr,  "mcam: Get1284DeviceID() failed\n" );
	return FALSE;
	}
fprintf(stderr,  "DeviceID = %s\n", deviceID );

if ( strstr( deviceID, "VLSI Vision Ltd" ) == NULL )
	{
	fprintf(stderr,  "mcam: DeviceID did not contain 'VLSI Vision Ltd'\n" );
	return FALSE;
	}
/*
considering bug of CPIA1_00
*/
//fprintf(stderr,  "Clearing Fatal and Cmd Status: " );
rtn = CPIA_ModifyCameraStatus( 3, 0, 0 );
rtn &= CPIA_ModifyCameraStatus( 4, 0, 0 );
if (!rtn) fprintf(stderr, "Send failed\n");
else fprintf(stderr, "OK\n");

fprintf (stderr, "Switching to Hi-Power mode ");
rtn = CPIA_GotoHiPower();
rtn &= CPIA_GetCameraStatus(\
	NULL, NULL, NULL, &fatal, &cmd, NULL, NULL, NULL );
if ( rtn && (!cmd) && (!fatal) ) fprintf(stderr,  "OK\n" );
/*
rtn = CPIA_GetCameraStatus( NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL );
*/

return( rtn );
}/* end function CheckForCPIA */

/*
Furthermore I disabled option 4 (goto LoPower). Choice of this option
results in hanging port since at least one of the commands in the main
loop (SetApcor, Get Exposure ...) seems to require HiPower for the
CPIA1_00  to be correctly transfered.
If you have a modified version of mcam  I would be pleased to test it
with my CPIA1_00.
Best regards
Heinz-Juergen
*/ 

BOOL DoSystemTest( void ) 
{
BOOL rtn, failed;
BYTE sys, vp, stream, fatal, cmd, vp_stat;
BYTE cam_setup0, cam_status;
WORD base, timeout;
BYTE port_type;
BYTE devicehi, devicelo;
BOOL I2CReadCmdAvailable;
		
failed = FALSE;

fprintf(stderr,  "Getting Firmware / VC version info   --- " );
rtn = CPIA_GetVersion( &gFW_Ver, &gFW_Rev, &gVC_Ver, &gVC_Rev );
if ( rtn ) 
	{
	fprintf(stderr,  "OK\n" );
	fprintf(stderr,  "  - Firmware = v%d-%02d    ", gFW_Ver, gFW_Rev );	
	fprintf(stderr,  "  VC = v%d-%02d\n", gVC_Ver, gVC_Rev );	
	if ( ((gFW_Ver==1)&&(gFW_Rev > 8)) || (gFW_Ver > 1) ) 
		{
		I2CReadCmdAvailable = TRUE;
		}
	else
		{
		I2CReadCmdAvailable = FALSE;
		}
	}
else
	{
	fprintf(stderr,  "FAILED\n" );
	fprintf(stderr,  "    Aborting System Test\n" );
	goto SystemTestFailed;
	}

fprintf(stderr,  "Clearing Fatal and Cmd status        --- " );
rtn = CPIA_ModifyCameraStatus( 3, 0, 0 ); 
rtn &= CPIA_ModifyCameraStatus( 4, 0, 0 ); 
if ( !rtn ) goto SendFailed;
else fprintf(stderr,  "OK\n" );

fprintf(stderr,  "Switching to Hi-Power mode           --- " );

rtn = CPIA_GotoHiPower();
rtn &= CPIA_GetCameraStatus(\
	&sys, &vp, &stream, &fatal, &cmd, NULL, &vp_stat, NULL ); 
if ( rtn && (!cmd) && (!fatal) ) fprintf(stderr,  "OK\n" );
else
	{
	fprintf(stderr,  "FAILED\n" );
	if ( fatal & VP_CTRL_FLAG ) fprintf(stderr,  "    VP_CTRL FatalError\n" );
	if ( fatal & CAPTURE_FLAG ) fprintf(stderr,  "    CAPTURE FatalError\n" );
	if ( cmd & VP_CTRL_FLAG ) fprintf(stderr,  "    VP_CTRL CmdError\n" );
	if ( cmd & CAPTURE_FLAG ) fprintf(stderr,  "    CAPTURE CmdError\n" );
	fprintf(stderr,  "    Aborting System Test\n" );
	goto SystemTestFailed;					
	}

CPIA_WriteVCReg( 0, 0x7F, 0x80 );

/* test dummy data transfer in 4 bit mode */
rtn = CLINK_GetPortInfo( &base, &port_type, &timeout );
rtn = CLINK_SetPortInfo( (port_type&COMPAT_PORT_TYPE), timeout );
fprintf(stderr,  "Uploading test data in Nibble mode   --- " );
rtn = UploadTestData( 256 );
if ( rtn ) fprintf(stderr,  "OK\n" );
else fprintf(stderr,  "FAILED\n" );

/* test dummy data transfer in ECP mode if possible*/
if ( port_type & ECP_PORT_TYPE )
	{
	rtn = CLINK_SetPortInfo( port_type, timeout );
	fprintf(stderr,  "Uploading test data in ECP mode      --- " );
	rtn = UploadTestData( 256 );
	if ( rtn ) fprintf(stderr,  "OK\n" );
	else fprintf(stderr,  "FAILED\n" );
	}
else
	{
	fprintf(stderr,  "Unable to upload test data in ECP mode\n" );
	}

/* disable AEC, ACB and wait */
fprintf(stderr,  "Checking for camera ...\n" );
rtn = CPIA_SetColourBalance( 3, 0, 0, 0 );
rtn = CPIA_SetExposure( 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
usleep(400000);
CPIA_ReadVPRegs( 56, 57, 0, 0, &cam_status, &cam_setup0, NULL, NULL );
if ( !(cam_status & 0x20) )
	{ 
	fprintf(stderr,  "  - Camera not detected by VP        --- FAILED\n" );
	failed = TRUE;
	}

/* check VP_Busy --> no camera OR unprogrammed EEPROM*/
if ( cam_status & 0x40 )
	{
	fprintf(stderr,  "  - VPBusy=HI, no camera/bad eeprom  --- FAILED\n" );
	failed = TRUE;
	}
  	
/* write camera register */	
fprintf(stderr,  "  - I2C write to sensor              --- " );
rtn = CPIA_ModifyCameraStatus( 4, 0, 0 );	/* clear the CmdError field */
rtn &= CPIA_SerialStart();
rtn &= CPIA_SerialWrite( 0x20 );
rtn &= CPIA_SerialWrite( 0x80 );	/* MSB set for auto-inc */
rtn &= CPIA_SerialStop();
rtn &= CPIA_GetCameraStatus(\
	NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
if ( !rtn || (cmd&SYSTEM_FLAG) )
	{
	fprintf(stderr,  "FAILED\n" );
	failed = TRUE;
	}
else
	{
	fprintf(stderr,  "OK\n" );
	}

if ( I2CReadCmdAvailable )
	{
	fprintf(stderr,  "  - I2C read from sensor             --- " );
	rtn = CPIA_SerialStart();
	rtn &= CPIA_SerialWrite( 0x21 );
	rtn &= CPIA_SerialRead( TRUE, &devicehi );	/* dummy read to get rid of index */
	rtn &= CPIA_SerialRead( TRUE, &devicehi );
	rtn &= CPIA_SerialRead( FALSE, &devicelo );
	rtn &= CPIA_SerialStop();
	if ( !rtn || (cmd&SYSTEM_FLAG) )
		{
		fprintf(stderr,  "FAILED\n" );
		failed = TRUE;
		}
	else
		{
		WORD id;
		fprintf(stderr,  "OK\n" );
		id = (devicehi<<4) + (devicelo>>4);
		fprintf(stderr,  "  - Sensor ID = %d  Mask Rev = %d\n", id, (devicelo&0x0F) );
		}
	}
else
	{
	fprintf(stderr,  "  - Firmware does not support I2C read\n" );
	} 

/* read EEPROM and verify checksum */
fprintf(stderr,  "Testing I2C to EEPROM ...\n" );
fprintf(stderr,  "  - I2C write to EEPROM              --- " );
rtn = CPIA_ModifyCameraStatus( 4, 0, 0 );	/* clear the CmdError field */
rtn &= CPIA_SerialStart();
rtn &= CPIA_SerialWrite( 0xA0 );
rtn &= CPIA_SerialWrite( 0x00 );
rtn &= CPIA_SerialStop();
rtn &= CPIA_GetCameraStatus(\
	NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
if ( !rtn || (cmd&SYSTEM_FLAG) )
	{
	fprintf(stderr,  "FAILED\n" );
	failed = TRUE;
	}
else
	{
	fprintf(stderr,  "OK\n" );
	}		

if ( I2CReadCmdAvailable )
	{
	fprintf(stderr,  "  - Verifying EEPROM checksum        --- " );
	rtn = TestEEPROM();
	if ( rtn ) fprintf(stderr,  "OK\n" );
	else
		{
		fprintf(stderr,  "FAILED\n" );
		failed = TRUE;
		}
	}
else
	{
	fprintf(stderr,  "  - Firmware does not support I2C read\n" );
	} 

CPIA_GotoLoPower();

if ( !failed ) return TRUE;
else return FALSE;

SystemTestFailed:
CPIA_GotoLoPower();
return FALSE;

SendFailed:
fprintf(stderr,  "\nUnable to send command --- aborting test\n\n" );
return FALSE;
}/* end function DoSystemTest */


void UploadAndSaveDRAM( void ) 
{
FILE *file;
int tmp;
char fs_filename[] = "fs.txt";
char rlb_filename[] = "rlb.txt";
DWORD pos;

file = fopen( fs_filename, "wb" );
if ( file == NULL )
	{
	fprintf(stderr,  "UploadAndSaveDRAM: Unable to open %s file\n", fs_filename );
	return;
	}
fprintf(stderr,  "UploadAndSaveDRAM: FS save file %s\n", fs_filename );

for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ ) gStreamBuffer[ pos ] = 0xFE;

if ( !CPIA_MACRO_UploadDRAM( 0, 1024, gStreamBuffer ) )
	{
	tmp = fprintf(stderr,  "UploadAndSaveDRAM: UploadDRAM failed\n" );
	goto EndUpload;
	}
fprintf(stderr,  "UploadAndSaveDRAM: FS uploaded\n" );
	
for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ )
	{
	fprintf(file, " %x", (BYTE )gStreamBuffer[pos] );
	if ( !(pos%1024) ) fprintf(stderr,  "." );
	}
fclose( file );
fprintf(stderr,  "\nUploadAndSaveDRAM: FS saved\n" );


file = fopen( rlb_filename, "wb" );
if ( file == NULL )
	{
	fprintf(stderr,  "UploadAndSaveDRAM: Unable to open %s file\n", rlb_filename );
	return;
	}
fprintf(stderr,  "UploadAndSaveDRAM: RLB save file %s\n", rlb_filename );

for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ ) gStreamBuffer[ pos ] = 0xFE;
if ( !CPIA_MACRO_UploadDRAM( 1024, 2048, gStreamBuffer ) )
	{
	fprintf(stderr,  "UploadAndSaveDRAM: UploadDRAM failed\n" );
	goto EndUpload;
	}
fprintf(stderr,  "UploadAndSaveDRAM: RLB uploaded\n" );

for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ )
	{
	fprintf(file, " %x", (BYTE )gStreamBuffer[pos] );
	if ( !(pos%1024) ) fprintf(stderr,  "." );
	}
fclose( file );
fprintf(stderr,  "\nUploadAndSaveDRAM: RLB saved\n" );

return;

EndUpload:
fclose( file );
return;
}/* end function UploadAndSaveDRAM */


BOOL UploadTestData( WORD DummyBlocks )
{
FILE *file;
int tmp;
char filename[] = "dummy.txt";
DWORD pos;
BYTE val;
int count;
BOOL upload_ok = TRUE;

for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ ) gStreamBuffer[ pos ] = 0xFE;

if ( !CPIA_MACRO_TestStream( 0, DummyBlocks, gStreamBuffer ) )
	{
	tmp = fprintf(stderr,  "UploadTestData: CPIA_TestStream() failed\n" );
	}
/*
file = fopen( filename, "wb" );
if ( file == NULL )
	{
	fprintf(stderr,  "UploadBlock: Unable to open %s file\n", filename );
	return;
	}
for( pos = 0; pos < (DWORD)256*DummyBlocks; pos++ )
	{
	fprintf( file, " %x", (BYTE )gStreamBuffer[pos] );
	if ( !(pos%1024) ) fprintf(stderr,  "." );
	}
fclose( file );
*/
val = 0; count = 0;
for( pos = 0; pos < ((DWORD)DummyBlocks * 256); pos++ )
	{
	if ( gStreamBuffer[ pos ] != val )
		{
		fprintf(stderr, "UploadTestData: gStreamBuffer at %lu   %d != %d\n",\
		pos, gStreamBuffer[ pos ], val );
		count++;
		if ( count > 4 ) break;
		upload_ok = FALSE;
		}
	val++;
	}

return( upload_ok );
}/* end function UploadTestData */


void DoUploadImage( void )
{
FILE *file;
int tmp;
char filename[] = "image.txt";
DWORD pos;
DWORD actual_bytes;

for( pos = 0; pos < STREAM_BUFFER_LEN; pos++ ) gStreamBuffer[ pos ] = 0xFE;

actual_bytes = 0;
if ( !CPIA_UploadStreamData(\
STREAM_BUFFER_LEN, gStreamBuffer, &actual_bytes, 2048 ) )
	{
	tmp = fprintf(stderr,  "UploadImage: UploadImageStream failed\n" );
	}
else
	{
	fprintf(stderr,  "UploadImage: Image uploaded %lu bytes\n", actual_bytes );
	}

file = fopen( filename, "wb" );
if ( file == NULL )
	{
	fprintf(stderr,  "UploadImage: Unable to open %s file\n", filename );
	return;
	}
/*
for( pos = 0; pos < (DWORD)actual_bytes; pos++ )
	{
*/
for( pos = 0; pos < (DWORD)STREAM_BUFFER_LEN; pos++ )
	{
	fprintf( file, " %x", (BYTE )gStreamBuffer[pos] );
	if ( !(pos%1024) ) fprintf(stderr,  "." );
	}
fclose( file );

return;
}/* end function DoUploadImage */


#define EEPROM_SIZE		128


BOOL TestEEPROM( void )
{
BOOL rtn, cmd;
BYTE eeprom[ EEPROM_SIZE ];
BYTE old_checksum;
BYTE calc_checksum;
int i;

rtn = CPIA_ModifyCameraStatus( 4, 0, 0 );	/* clear the CmdError field */
rtn &= CPIA_SerialStart();
rtn &= CPIA_SerialWrite( 0xA0 );
rtn &= CPIA_SerialWrite( 0x00 );
rtn &= CPIA_SerialStop();
rtn &= CPIA_GetCameraStatus( NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
if ( !rtn || (cmd&SYSTEM_FLAG) )
	{
	return FALSE;
	}

rtn = CPIA_ModifyCameraStatus( 4, 0, 0 );	/* clear the CmdError field */

for( i = 0; i < EEPROM_SIZE-1; i++ ) eeprom[ i ] = 0x80;

rtn &= CPIA_SerialStart();
rtn &= CPIA_SerialWrite( 0xA1 );
for( i = 0; i < EEPROM_SIZE-1; i++ )
	{
	rtn &= CPIA_SerialRead( TRUE, &eeprom[i] );
	}
rtn &= CPIA_SerialRead( FALSE, &eeprom[i] );
rtn &= CPIA_SerialStop();
rtn &= CPIA_GetCameraStatus( NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
if ( !rtn || (cmd&SYSTEM_FLAG) ) return FALSE;

/* now check that the checksum is correct 	*/
old_checksum = (int)eeprom[ 0x1F ];
calc_checksum = 0;
eeprom[ 0x1F ] = 0;
for( i = 0; i < EEPROM_SIZE-1; i++ )
	{
	calc_checksum += eeprom[i];
	}
//fprintf(stderr,  "\nold_checksum = %d    calc_checksum = %d\n", (int)old_checksum, (int)calc_checksum );

if ( old_checksum != calc_checksum ) return FALSE;
else return TRUE;
}/* end function TestEEPROM */


int wait_key()
{
int c;

/* read a user key */	
system("stty raw");
c = getc(stdin);
system("stty sane");
return 1;
}


int console_status()
{
int a, c;
fd_set read_file_descr;
struct timeval timeout;

/* this macro initializes the file descriptor read_file_descr to to be the empty
/* set */
FD_ZERO(&read_file_descr);

/* this macro adds fileno(stdin) to the file descriptor read_file_descr */
FD_SET(fileno(stdin), &read_file_descr);

timeout.tv_sec = 0;
timeout.tv_usec = 0;
/* int FD_SETSIZE  macro is maximum number of filedescriptors that
/* fd_set can hold */
/* function select waits for specified filedescr. to have a signal */
/* last argument struct timeval *timeout */
a = select(FD_SETSIZE, &read_file_descr, NULL, NULL, &timeout);
if(a == -1)/* 0 is timeout, -1 error (in errno), 1 = data */
	{
	if(debug_flag) fprintf(stderr, "select returned -1 error\n");
	return 0;
	}
if(a == 0)
	{
	if(debug_flag) fprintf(stderr, "select returned 0 timeout\n");
	return 0;
	}
if(a == 1)
	{
	if(debug_flag) fprintf(stderr, "SELECT returned=%d input\n", a);
	return 1;
	}

/* test if user has data */
if( FD_ISSET(fileno(stdin), &read_file_descr) )
	{
	c = getc(stdin);
	if(debug_flag)fprintf(stderr, "USER KEY=%d\n", c);
/*	FD_CLR(fileno(stdin), &read_file_descr);*/
	}
}/* end function console_status */


int get_port_base_address(int *address)
{
*address = port_address;

return 1;
}/* end function get_port_base_address */


#ifdef USE_X
int read_and_display_frame(char *filename, int argc, char **argv)
{
int c, i;
FILE *fptr;
unsigned char *ucptr;
FILE *pptr;
int xsize, ysize;
int size;
struct stat *statptr;
int x_times_y;

/* open file */
fptr = fopen(filename, "r");
if(! fptr)
	{
	fprintf(stderr, "Cannot open file %s for read\n", filename);
	strerror(errno);
	return 0;
	}

/* get file length */
statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr)
	{
	fprintf(stderr,\
	"read_and_display_frame(): could allocate space fro structure stat\n");
	return 0;
	}
fstat(fileno(fptr), statptr);

/* create a window to get the color depth, if not manual color depth */
if(color_depth_mode == 0)
	{
	if(! window_open_flag)
		{
		openwin(argc, argv, 352, 288);
		current_window_size = size;
		window_open_flag = 1;
		}
	}

x_times_y = statptr -> st_size  / (wingetdepth() / 8);

if(debug_flag)
	{
	fprintf(stderr,
	"read_and_display_frame():\n\
	file_size=%d x_times_y=%d\n",  statptr -> st_size, x_times_y);
	}

if( (x_times_y / 352) > 200)
	{
	xsize = 352;
	ysize = 288;
	size = BIG;
	}
else
	{
	xsize = 176;
	ysize = 144;
	size = SMALL;
	}

if(debug_flag)
	{
	fprintf(stderr,\
	"read_and_display_frame(): xsize=%d ysize=%d\n", xsize, ysize);
	}

/* free structure stat */
free(statptr);

/* create an xwindows display */
if(window_open_flag)
	{
	if(size != current_window_size)
		{
/*		closewin();*/
/*		resize_window(xsize, ysize);*/
		openwin(argc, argv, xsize, ysize);
		current_window_size = size;
		}
	}
else
	{
	openwin(argc, argv, xsize, ysize);
	current_window_size = size;
	window_open_flag = 1;
	}

ucptr = (unsigned char *)getbuf();
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	if(c == EOF) break;
	*ucptr = c;
	ucptr++;
	}
fclose(fptr);

putimage(xsize, ysize);

return 1;
}/* end function read_and_display_frame */
#endif /* USE_X */


int make_jpeg_in_buffer(int *bytes_read)
{
int a, c, i;
DWORD actual_bytes;
BYTE stream;
unsigned char *ucptr;
char temp[1024];
FILE *fptr;
FILE *pptr;
int compression_flag;
int size;
int xsize, ysize;
float compression_factor;

if(debug_flag)
	{
	fprintf(stderr, "make_jpeg_in_buffer() arg none\n");
	}

/* get x and y size from frame buffer header */
if (gStreamBuffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 3;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 2;
	}
else
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 2;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "make_jpeg_in_buffer(): coding xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
	
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "make_jpeg_in_buffer(): mpeg xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;
	}

/* create mcam-ppm.ppm file */
sprintf(temp, "%s/mcam-ppm.ppm", frame_dir);
fptr = fopen(temp, "w");
if(!fptr)
	{
	fprintf(stderr,\
	"make_jpeg_in_buffer(): cannot open file %s for write\n", temp);
	strerror(errno);
	return 0;
	}
	
compression_flag = DataBuffer[HEADER_SIZE + 28];

if(! gstream_to_ppm(\
	xsize, ysize, gStreamBuffer, *bytes_read, fptr, compression_flag) )
	{
	return 0;
	}
sprintf(temp,\
"cjpeg -quality %d -outfile %s/mcam-jpeg.jpg %s/mcam-ppm.ppm 1> /dev/null 2> /dev/null",\
coding_quality, frame_dir, frame_dir);
pptr = popen(temp, "r");
pclose(pptr);

if(debug_flag)
	{
	fprintf(stderr, "make_jpeg_in_buffer(): mpeg past pclose()\n");
	}

/* bitstream now in frame_dir/mcam-mpeg.out or frame_dir/mcam-jpeg.jpg */

/* bitstream to gStreamBuffer + 64 */

sprintf(temp, "%s/mcam-jpeg.jpg", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"make_jpeg_in_buffer(): could not open file %s for read\n", temp);
	strerror(errno);
	return 0;
	}	

i = 0;
ucptr = gStreamBuffer + 64;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}	
	if(c == EOF)
		{
		fclose(fptr);
		break;
		}
	*ucptr = c;
	ucptr++;
	i++;
	}
	
/* header + data */
*bytes_read = i + 64;
	
if(debug_flag)
	{
	fprintf(stderr,"make_jpeg_in_buffer(): jpeg bytes_read=%d\n", *bytes_read);
	}

return 1;
}/* end function make_jpeg_in_buffer */


int make_mpeg_in_buffer(int *bytes_read)
{
int a, c, i;
DWORD actual_bytes;
BYTE stream;
unsigned char *ucptr;
char temp[1024];
FILE *fptr;
FILE *pptr;
int compression_flag;
int size;
int xsize, ysize;
float compression_factor;

if(debug_flag)
	{
	fprintf(stderr, "make_mpeg_in_buffer() arg none\n");
	}

/* mpeg coding here */

/*
					IMPORTANT COPYRIGHT NOTICE README
If mpeg coding use the Berkely mpeg2encode to create a bit of a bitstream
with as input a .ppm file.
This because it does not accept gstream, but accepts .ppm (not mofified yet).
It is of cause cheating to code only ONE frame into ONE piece of bitstream,
but since the Berkely decoder cannot parse the bitstream (?), clients
connecting 'later' would not be able to decode the data anyways.
The artifacts you get in the mpeg coding / decoding process SHOULD perhaps
be less if more frames are used (taken into account), but now I am going
for the incredible compression rate (127 x or so) I have measured on my test
frames (14 frames of 300kB in 36056 bitstream out) , and 44 x or so for a
single frame (203618 (gstream) / 4602 bitstream out).
So I will think about streaming :)
For now only single frame compression.
This does not take full advantage of the mpeg coding system however.
Also the communication with the Berkely software mpeg coder / decoder is
via temp files.
The penalty for this in speed is not that severe because of caching.
I cannot integrate the Berkely decoder because it is not GPL, and for non
commercial use only.
GPL requires it to be a separate program.
Wish they GPL'd it, then I could integrate it.
Please in using mcam with the Berkely coder/decoder for non commercial
purposes only, see the Berkely Makefile.
mcam itself is GPL, but if you use the decoder.... :)
SO, again, you CANNOT integrate the Berkely coder/decoder or any modification
of it in mcam, as mcam has to stay GPL.
It is forbidden to remove this notice from the code. 
*/
if(debug_flag)
	{
	fprintf(stderr, "make_mpeg_in_buffer(): starting coding\n");
	}

/* get x and y size from frame buffer header */
if (gStreamBuffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 3;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 2;
	}
else
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 2;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "make_mpeg_in_buffer(): coding xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
	
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "make_mpeg_in_buffer(): mpeg xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;
	}

/* create mcam-ppm.ppm file */
sprintf(temp, "%s/mcam-ppm.ppm", frame_dir);
fptr = fopen(temp, "w");
if(!fptr)
	{
	fprintf(stderr,\
	"make_mpeg_in_buffer(): cannot open file %s for write\n", temp);
	strerror(errno);
	return 0;
	}
	
compression_flag = DataBuffer[HEADER_SIZE + 28];

if(! gstream_to_ppm(\
xsize, ysize, gStreamBuffer, *bytes_read, fptr, compression_flag) )
	{
	return 0;
	}

if(size == BIG)
	{
	/* uses as input mcam-ppm.ppm as specified in big.par */
	sprintf(temp,\
	"mpeg2encode %s/big.par %s/mcam-mpeg.out >&/dev/null",\
	frame_dir, frame_dir);
	}
else /* small */
	{
	/* uses as input mcam-ppm.ppm as specified in small.par */
	sprintf(temp,\
	"mpeg2encode %s/small.par %s/mcam-mpeg.out >&/dev/null",\
	frame_dir, frame_dir);
	}
if(debug_flag)
	{
	fprintf(stderr, "make_mpeg_in_buffer(): mpeg executing=%s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);
if(debug_flag)
	{
	fprintf(stderr, "make_mpeg_in_buffer(): mpeg past pclose()\n");
	}

/* bitstream now in frame_dir/mcam-mpeg.out or frame_dir/mcam-mpeg.jpg */

/* bitstream to gStreamBuffer + 64 */

sprintf(temp, "%s/mcam-mpeg.out", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"make_mpeg_in_buffer(): mpeg could not open file %s for read\n", temp);
	strerror(errno);
	return 0;
	}	

i = 0;
ucptr = gStreamBuffer + 64;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}	
	if(c == EOF)
		{
		fclose(fptr);
		break;
		}
	*ucptr = c;
	ucptr++;
	i++;
	}
	
/* header + data */
*bytes_read = i + 64;
	
if(debug_flag)
	{
	fprintf(stderr,"make_mpeg_in_buffer(): mpeg bytes_read=%d\n", *bytes_read);
	}
	
return 1;
}/* end function make_mpeg_in_buffer */


int get_a_frame(int *bytes_read, int l_coding_mode)
{
int a, c, i;
long usec_delay;
float time_trying;
DWORD actual_bytes;
BYTE stream;

if(debug_flag)
	{
	fprintf(stderr,\
	"get_a_frame(): arg l_coding_mode=%d\n",\
	l_coding_mode); 
	}

/* send the grab a frame (into the camera DRAM) command */
if(! CPIA_GrabFrame(0) )
	{
	fprintf(stderr, "get_a_frame(): CPIA_GrabFrame failed\n" );
	return 0;
	}

/* wait for the camera to tell us that it is ready to send the data */
/* timeout after .. status checks */

usec_delay = 100000;/* .1 seconds */
time_trying = 0.0;
while(1)
	{
	/* wait for stream ready */
	if(! CPIA_GetCameraStatus(\
	NULL, NULL, (BYTE *)&stream, NULL, NULL, NULL, NULL, NULL ) )
 		{
		fprintf(stderr,  "get_a_frame(): CPIA_GetCameraStatus failed\n" );
		return 0;
		}
	if(debug_flag)
		{
		fprintf(stderr,\
		"get_a_frame(): looping waiting for camera stream ready\n");
		}
	/* wait .1 second, so we have a defined timeout */
	usleep(usec_delay);

	time_trying += .000001  * (float)usec_delay;
	if(time_trying >= MAX_WAIT_TIME)
		{
		fprintf(stderr, "get_a_frame(): timeout status ready\n");
		return 0;
		}

	if(stream == STREAM_READY) break;
	}/* end while wait for stream ready */
	
/* upload the data stream (from camera to host) */
/* bytes_read already a pointer */
if(! CPIA_UploadStreamData(\
STREAM_BUFFER_LEN, gStreamBuffer, (DWORD *)bytes_read, 1024 ) )
	{
	fprintf(stderr,  "get_a_frame(): CPIA_UploadStreamData failed\n" );
	return 0;
	}

if(debug_flag)
	{
	fprintf(stderr, "get_a_frame(): length buffer=%d bytes_read=%d\n",\
	STREAM_BUFFER_LEN, *bytes_read);
	}

/* add server clock here */
if(show_time_flag)
	{
	if(! add_clock(gStreamBuffer) ) return 0;
	}

/* add server text here, nothing happens if strlen text is zero */
if(! text_to_gstream(\
	screen_text, text_x, text_y,\
	text_fr, text_fg, text_fb,\
	text_br, text_bg, text_bb,\
	gStreamBuffer)\
	) return 0;

switch(l_coding_mode)
	{
	case JPEG_CODING:
		if(! make_jpeg_in_buffer(bytes_read) ) return 0;
		break;
	case  MPEG2_CODING:
		if(! make_mpeg_in_buffer(bytes_read) ) return 0;
		break;
	case WAVELET_CODING:
		if(! make_wavelet_in_buffer(bytes_read) ) return 0;
		break;
	default: /* no coding */
		/* leave buffer as it is */
		break;
	}/* end switch l_coding_mode */

return 1;
}/* end function get_a_frame */


int convert_jpeg_in_buffer_to_gstream(int *bytes_in_buffer)
{
int i;
FILE *fptr;
char temp[1024];
FILE *pptr;

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_jpeg_in_buffer_to_gstream(): arg none\n");
	}

/* write data in gStreamBuffer + 64 to temp file */
sprintf(temp, "%s/mcam-jpeg.in", frame_dir);

fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"convert_jpeg_in_buffer_to_gstream():\n\
	cannot open file %s for write\n",\
	temp);
	strerror(errno);
	return 0;
	}
for(i = 0; i < *bytes_in_buffer; i++)
	{
	while(1)
		{
		errno = 0;
		fputc(gStreamBuffer[64 + i], fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	}
fclose(fptr);

if(debug_flag)
	{
	fprintf(stderr,\
"convert_jpeg_in_buffer_to_gstream(): jpeg written dataBuffer + 64 (%d bytes) to %s\n",\
	*bytes_in_buffer, temp);
	}	
	
/*
tell the modified decoder to decode mpeg bitstream file or jpeg file to
gstream format
*/

sprintf(temp, "djpeg -pnm  %s/mcam-jpeg.in > %s/mcam-jpeg.out",\
frame_dir, frame_dir);
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_jpeg_in_buffer_to_gstream(): jpeg executing %s\n", temp);
	}
pptr = popen(temp, "r");		
pclose(pptr);	
	
/* data now in file frame_dir/mcam-gstream or frame_dir/mcam-jpeg.out*/
		
/* read gstream file into dataBuffer + 64; */

/*
we have the file mcam-jpeg.out in ppm format
need to convert to gstream format in gStreamBuffer + 64.
*/

/* open the ppm format file */
sprintf(temp, "%s/mcam-jpeg.out", frame_dir);

if(! ppm_to_gstream(temp, gStreamBuffer + 64, bytes_in_buffer) )
	{
	return 0;
	}
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_jpeg_in_buffer_to_gstream(): jpeg ppm bytes_in_buffer=%d\n",\
	*bytes_in_buffer);
	}	

/*
data in gStreamBuffer + 64, which is DataBuffer + 128 in gstream format
mcam header (64) cpia header (64) data (bytes_in_buffer).
*/

/*
NOTE IMPORTANT: if mpeg and compressed, then gstream is uncompressed
*/
/* signal to convert() not compressed */
gStreamBuffer[COMP_ENABLE] = NOT_COMPRESSED;

return 1;
}/* end function convert_jpeg_in_buffer_to_gstream */


int convert_mpeg_in_buffer_to_gstream(int *bytes_in_buffer)
{
int a, i;
FILE *fptr;
char temp[1024];
FILE *pptr;
unsigned char *ucptr;

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream(): arg none\n");
	}

/* if data in mpeg bitstream format call external decoder */

/* write data in gStreamBuffer + 64 to temp file */
sprintf(temp, "%s/mcam-mpeg.in", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream(): cannot open file %s for write\n",\
	temp);
	strerror(errno);
	return 0;
	}
for(i = 0; i < *bytes_in_buffer; i++)
	{
	while(1)
		{
		errno = 0;
		fputc(gStreamBuffer[64 + i], fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	}
fclose(fptr);

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream():\n\
	written dataBuffer + 64 (%d bytes) to %s\n",\
	*bytes_in_buffer, temp);
	}	
	
/*
tell the modified decoder to decode mpeg bitstream file to gstream format
*/

sprintf(temp, "p2dec -b %s/mcam-mpeg.in -o1 %s/mcam-gstream >&/dev/null",\
frame_dir, frame_dir);
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream(): executing %s\n", temp);
	}
pptr = popen(temp, "r");		
pclose(pptr);	
	
/* data now in file frame_dir/mcam-gstream */
		
/* read gstream file into dataBuffer + 64; */

sprintf(temp, "%s/mcam-gstream.gs", frame_dir);
fptr = fopen(temp, "r");
if(!fptr)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream(): cannot open file %s for read\n",\
	temp);
	strerror(errno);
	return 0;
	}

ucptr = gStreamBuffer + 64;
*bytes_in_buffer = 0;
while(1)
	{
	a = read(fileno(fptr), ucptr, STREAM_BUFFER_LEN - *bytes_in_buffer);
	/* test for error */
	if(a < 0)
		{
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		
		perror("convert_mpeg_in_buffer_to_gstream():");
		}
	/* test for EOF */
	if(a == 0)
		{
		fclose(fptr);
		break;
		}
	*bytes_in_buffer += a;
	ucptr += a;
	}/* end while gstream file to gStreamBuffer */

/* length data read now in bytes_read */
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_mpeg_in_buffer_to_gstream(): decoded bytes_in_buffer=%d\n",\
	*bytes_in_buffer);
	}	

/*
data in gStreamBuffer + 64, which is  DataBuffer + 128 in gstream format
mcam header (64) cpia header (64) data (bytes_in_buffer).
*/

/*
NOTE IMPORTANT: if mpeg and compressed, then gstream is uncompressed
*/
/* signal to convert() not compressed */
gStreamBuffer[COMP_ENABLE] = NOT_COMPRESSED;

return 1;
}/* end function convert_mpeg_in_buffer_to_gstream */


int display_a_frame(\
int argc, char **argv,\
int bytes_in_buffer, int l_coding_mode, int compression_flag)
{
int a, i;
FILE *fptr;
char temp[1024];
unsigned char *ucptr;
BYTE __huge *compression_buffer;
int compressed_length, decompresed_length;
FILE *pptr;
int xbuf_size;
int xsize, ysize;
int size;
static int old_size;
char temp_filename_1[L_tmpnam];
char temp_filename_2[L_tmpnam];
char *ptr;
int change_flag;
BYTE __huge *ppm_buffer;

if(debug_flag)
	{
	fprintf(stderr,\
	"display_a_frame():\n\
	arg argc argv bytes_in_buffer=%d l_coding_mode=%d compression_flag=%d\n",\
	bytes_in_buffer, l_coding_mode, compression_flag);
	}

/* argument check */
if(bytes_in_buffer == 128) return 0; /* grab_a_frame returned 0,
										we have only the header
										*/

/* get x and y size from frame buffer header */
if (gStreamBuffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize = (gStreamBuffer[COL_END] - gStreamBuffer[COL_START]) << 3;
	ysize = (gStreamBuffer[ROW_END] - gStreamBuffer[ROW_START]) << 2;
	}
else
	{
	xsize =	(gStreamBuffer[COL_END] - gStreamBuffer[COL_START]) << 2;
	ysize = (gStreamBuffer[ROW_END] - gStreamBuffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "display_a_frame(): xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
					
/*
make it possible to start the client, so in case of 'could not get a
frame' we can send commands to the server.
If xsize and/or ysize would be zero, x would exit.
Now we just assume a big picture.
*/
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "display_a_frame(): xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;	
	}

#ifdef USE_X
/* create an xwindows display */
if(window_open_flag)
	{
	if(size != current_window_size)
		{
/*		closewin();*/
		openwin(argc, argv, xsize, ysize);
/*		resize_window(xsize, ysize);*/
		current_window_size = size;
		}
	}
else
	{
	openwin(argc, argv, xsize, ysize);
	current_window_size = size;
	window_open_flag = 1;
	}
#endif // USE_X

/*
data now in gStreamBuffer + 64
normally this would be in format:
linelength low, linelength high, YUYV YUYV... EndOfLine (0xfd)
EndOfFrame (0xff 0xff 0xff 0xff)
Here called gstream format.

If l_coding_mode is JPEG_CODING it would be in jpeg formmat,
if MEPG2_CODING in mpeg bitstream format,
(as created by the modified Berkely mpeg2encode) EVEN IF compression was on,
since gstream_to_ppm uncompressed the data, before mpeg coding),
and if WAVELET_CODING is special wavelet (y+u+v) format.
or jpeg format.
*/

switch(l_coding_mode)
	{
	case JPEG_CODING:
		if(! convert_jpeg_in_buffer_to_gstream(&bytes_in_buffer) ) return 0;
		break;
	case MPEG2_CODING:
		if(! convert_mpeg_in_buffer_to_gstream(&bytes_in_buffer) )  return 0;
		break;
	case WAVELET_CODING:
		if(! convert_wavelet_in_buffer_to_gstream(&bytes_in_buffer) ) return 0;
		break;
	default: /* no coding */
		break;
	}/* end switch l_coding_mode */

/* 1 = compressed, else 0 */
compression_flag = gStreamBuffer[COMP_ENABLE];

if(special_effects_flag)
	{
	/* apply special effects to gStreamBuffer (YUV processing) */
	p_effect(gStreamBuffer, bytes_in_buffer);
	}

/* TEST new routines */
/*
if(test_flag)
	{
	test_test(gStreamBuffer, bytes_in_buffer,\
	y_difference, u_difference, v_difference, treshhold);
	}

*/
if(store_only_on_change_flag)
	{
	/* test for changes in picture */
	detect_change(\
	gStreamBuffer, bytes_in_buffer, &change_flag,\
	y_difference, u_difference, v_difference, treshhold);
	if(debug_flag)
//	if(test_flag)
		{
		if(change_flag)
			{
			fprintf(stderr,\
			"WHOOPY PICTURE HAS CHANGED treshhold=%d y_difference=%d\n",\
			treshhold, y_difference);
			}
		else
			{
			fprintf(stderr, "No change in picture detected\n");
			}
		}
	}/* end if store_only_on_change_flag */
	
/* add clock to gstreambuffer */
if(show_time_flag)
	{
	if(! add_clock(gStreamBuffer) ) return 0;
	}

/* add server text here, noting happens if strlen text is zero */
if(! text_to_gstream(\
	screen_text, text_x, text_y,\
	text_fr, text_fg, text_fb,\
	text_br, text_bg, text_bb,\
	gStreamBuffer)\
	) return 0;

#ifdef USE_X
/* convert the data in the frame buffer to X format in getbuf() */
xbuf_size = convert(\
gStreamBuffer, bytes_in_buffer, (unsigned char *)getbuf(), wingetdepth() );
if(debug_flag)
	{
	fprintf(stderr, "convert returned=%d\n", xbuf_size);
	}
#endif /* USE_X */

if( (!store_only_on_change_flag) ||\
(store_only_on_change_flag && change_flag) )
	{
	/* save to frame with handshake if requested */
	if(output_to_frame_flag)
		{
		/*
		wait for semaphore file deleted (mcam.ppm has been read),
		so we can create a new mcam.ppm.
		*/
		while(1)
			{
			/* try to open the semaphore file */
			sprintf(temp, "%s/mcam.ppm.ready", frame_dir);
			fptr = fopen(temp, "r");
			
			/* if file does not exists, we can create a new mcam.ppm */
			if(! fptr) break;
	
			/* close the semaphore file */
			fclose(fptr);
			
			/* sleep for a tenth of a second, to reduce processor load */
			usleep(100000);
			}/* end while test if semaphore file exists */
	
		sprintf(temp, "%s/mcam.ppm", frame_dir);
		fptr = fopen(temp, "w");
		if(!fptr)
			{
			fprintf(stderr,\
			"Save frames: cannot open file %s for write\n",\
			temp);
			strerror(errno);
			return 0;
			}
		
		if(! gstream_to_ppm(\
		xsize, ysize, gStreamBuffer, bytes_in_buffer, fptr, compression_flag) )
			{
			return 0;
			}
	
		/*
		create the semaphore file, to indicate picture should be read.
		The program that reads mcam.ppm should delete the semaphore file!!!
		*/
		sprintf(temp, "%s/mcam.ppm.ready", frame_dir);
		fptr = fopen(temp, "w");
		fclose(fptr); 
	
		}/* end if output_to_frame_flag */
	
	/* save to disk if user requests it */
	if(save_frames_flag)
		{
		if(save_frames_format == 2) /* mjpegtools yuv4mpeg to stderr */ 
			{
			if(! gstream_to_yuv(\
			xsize, ysize, gStreamBuffer, bytes_in_buffer, compression_flag) )
				{
				fprintf(stderr, "mcam: yuv output: gstream_to_yuv() failed.\n");

				return 0;
				}
			}
		else if(save_frames_format == 1)/* ppm */
			{
			sprintf(temp, "%s/frame.%d.ppm", frame_dir, frame_count);
			fptr = fopen(temp, "w");
			if(!fptr)
				{
				fprintf(stderr,\
				"Save frames: cannot open file %s for write\n",\
				temp);
				strerror(errno);
				return 0;
				}
	
			if(! gstream_to_ppm(\
			xsize, ysize, gStreamBuffer, bytes_in_buffer, fptr, compression_flag) )
				{
				return 0;
				}
			}/* end if ppm format */
#ifdef USE_X
		else /* write xbuffer */
			{
			sprintf(temp,\
			"%s/frame.%d", frame_dir, frame_count);
			fptr = fopen(temp, "w");
			if(!fptr)
				{
				fprintf(stderr,\
				"Save frames: cannot open file %s for write\n",\
				temp);
				strerror(errno);
				return 0;
				}
			ucptr = (unsigned char *)getbuf();
			for(i = 0; i < xbuf_size; i++)
				{
				while(1)
					{
					errno = 0;
					fputc(*ucptr, fptr);
					if(errno == EAGAIN) continue;
					if(errno == EINTR) continue;
					break;
					}
				ucptr++;
				}
			fclose(fptr);
			}/* end if write xbuffer */
#endif /* USE_X */
		frame_count++;
		}/* end save frames flag */

	}/* end if store all, or store on change and change detected */

#ifdef USE_X
/* converted data to X display */
putimage(xsize, ysize);
#endif /* USE_X */

/* wait user specified time */
if(frame_delay) usleep(frame_delay * 1000);

return 1;
}/* end function display_a_frame */


int get_color_depth()
{
#ifdef USE_X
return wingetdepth();
#else
return color_depth;
#endif
}/* end function get_color_depth */


int get_x11_color_depth()
{
int a;

a = -1;

#ifdef USE_X
a = get_x11_bpp();
#endif
return a;
}/* end function get_x11_color_depth */


int set_sensor_frames_per_second(int a, float *using_fps)
{
/* set frames per second developer pdf page 17 */
int sensor_base_rate;
int sensor_clock_divisor;
float fps;

if(debug_flag)
	{
	fprintf(stderr,\
	"set_sensor_frames_per_second(): arg a=%d\n", a);
	}

fps = (float) a;

if(fps >= 30.0)
	{
	sensor_base_rate = 1;
	sensor_clock_divisor = 0;
	*using_fps = 30.0;
	}
else if(fps >= 25.0)
	{
	sensor_base_rate = 0;
	sensor_clock_divisor = 0;
	*using_fps = 25.0;
	}
else if(fps >= 15.0)
	{
	sensor_base_rate = 1;
	sensor_clock_divisor = 1;
	*using_fps = 15.0;
	}
else if(fps >= 12.5)
	{
	sensor_base_rate = 0;
	sensor_clock_divisor = 1;
	*using_fps = 12.5;
	}
else if(fps >= 7.5)
	{
	sensor_base_rate = 1;
	sensor_clock_divisor = 2;
	*using_fps = 7.5;
	}
else if(fps >= 6.25)
	{
	sensor_base_rate = 0;
	sensor_clock_divisor = 2;
	*using_fps = 6.25;
	}
else if(fps >= 3.75)
	{
	sensor_base_rate = 1;
	sensor_clock_divisor = 3;
	*using_fps = 3.75;
	}
else if(fps >= 3.125)
	{
	sensor_base_rate = 0;
	sensor_clock_divisor = 3;
	*using_fps = 3.125;
	}
else /* < 3.125 */
	{
	sensor_base_rate = 0;
	sensor_clock_divisor = 3;
	*using_fps = 3.125;
	}
			
/* set frames per second */
CPIA_SetSensorFPS(sensor_clock_divisor, sensor_base_rate);

/* return OK */
return 1;
} /* end function set_sensor_frames_per_second */

